@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -23,6 +23,8 @@ module type GRAPH =
     type cfg
     val predecessors:     cfg -> node -> node list
     val successors:       cfg -> node -> node list
+    val direct_predecessors:     cfg -> node -> node list
+    val direct_successors:       cfg -> node -> node list
     val extract_is_loop : cfg -> node -> bool
     val print_node :      node -> unit
     val size :            cfg -> int
@@ -33,10 +35,9 @@ module type GRAPH =
 module OGRAPHEXT_GRAPH :
   sig
     type node = int
-    type cfg = (string, unit) Ograph_extended.ograph_mutable
-    val predecessors :
-      < predecessors : 'a -> < tolist : ('b * 'c) list; .. >; .. > ->
-      'a -> 'b list
+    module G : Ograph_extended.S
+    type cfg = string G.ograph_mutable
+    val predecessors : cfg -> node -> node list
     val print_node : node -> unit
   end
 
