@@ -44,9 +44,8 @@ LD_LIBRARY_PATH=$COCCINELLE_HOME:$LD_LIBRARY_PATH ; export LD_LIBRARY_PATH
 
 # To find .py files like the one in python/coccib
 echo adding $COCCINELLE_HOME/python to PYTHONPATH
-PYTHONPATH=$COCCINELLE_HOME/python:PYTHONPATH	; export PYTHONPATH
+PYTHONPATH=$COCCINELLE_HOME/python:$PYTHONPATH	; export PYTHONPATH
 
 fi
 
 export PATH=$COCCINELLE_HOME:$PATH
-
