@@ -288,6 +288,8 @@ and fullType = typeQualifier * typeC
           p_register: bool wrap;
           p_type: fullType;
           p_attr: attribute list;
+          p_midattr: attribute list;
+          p_endattr: attribute list;
         }
         (* => (bool (register) * fullType) list * bool *)
 
@@ -298,7 +300,13 @@ and typeQualifierbis = {const: bool; volatile: bool}
 (* gccext: cppext: *)
 and attribute = attributebis wrap
   and attributebis =
-    | Attribute of string
+    | Attribute of attr_arg
+    | GccAttribute of attr_arg
+
+and attr_arg = attr_arg_bis wrap
+  and attr_arg_bis =
+    | MacroAttr of string
+    | MacroAttrArgs of string * argument wrap2 (* , *) list
 
 (* ------------------------------------------------------------------------- *)
 (* C expression *)
@@ -577,6 +585,7 @@ and declaration =
          v_storage: storage;
          v_local: local_decl; (* cocci: *)
          v_attr: attribute list; (* gccext: *)
+         v_midattr: attribute list; (* gccext: *)
          v_endattr: attribute list; (* gccext: *)
        }
      and v_init =
@@ -627,6 +636,7 @@ and definition = definitionbis wrap (* ( ) { } fakestart sto *)
     f_storage: storage;
     f_body: compound;
     f_attr: attribute list; (* gccext: *)
+    f_endattr: attribute list; (* gccext: *)
     f_old_c_style: declaration list option;
   }
   (* cppext: IfdefFunHeader TODO *)
@@ -821,7 +831,7 @@ and metavars_binding = (Ast_cocci.meta_name, metavar_binding_kind) assoc
   | MetaStmtListVal  of statement_sequencable list * stripped
   | MetaDParamListVal of (string wrap) wrap2 list
   | MetaFmtVal       of string_format
-  | MetaAttributeVal of attribute
+  | MetaAttrArgVal   of attr_arg
   | MetaFragListVal  of string_fragment list
   | MetaAssignOpVal  of assignOp
   | MetaBinaryOpVal  of binaryOp
@@ -1327,8 +1337,19 @@ let fieldname_of_fieldkind fieldkind =
 
 
 let s_of_attr attr =
+  let get_attr_name = function
+    MacroAttr s, _ -> s
+  | MacroAttrArgs (attr,args), ii ->
+      let (_,lp,rp) = Common.tuple_of_list3 ii in
+      attr ^ (str_of_info lp) ^ "args... (not supported to show)" ^ (str_of_info rp) in
+  let get_attr = function
+    (Attribute a, ii) ->
+      get_attr_name a
+  | (GccAttribute a, ii) ->
+      let (attr_,_,_,_,_) = Common.tuple_of_list5 ii in
+      str_of_info attr_ ^ "((" ^ (get_attr_name a) ^ "))" in
   attr
-  +> List.map (fun (Attribute s, ii) -> s)
+  +> List.map get_attr
   +> String.concat ","
 
 
