@@ -1116,7 +1116,14 @@ let rec find_macro_paren xs =
   | [] -> ()
 
   (* attribute *)
-  | PToken ({tok = Tattribute ii} as id)
+  (*
+   * Current coccinelle supports only __attribute__ or __attribute
+   * with one argument. Since the current implementation is
+   * incomplete, this code is commented out just in case.
+   * In the future, we will add support for gcc attribute with
+   * multiple arguments.
+   *)
+(*| PToken ({tok = Tattribute ii} as id)
     ::Parenthised (xxs,info_parens)
     ::xs
      ->
@@ -1126,7 +1133,16 @@ let rec find_macro_paren xs =
         iter_token_paren (TV.set_as_comment Token_c.CppAttr);
       TV.set_as_comment Token_c.CppAttr id;
       find_macro_paren xs
-
+*)
+  | PToken ({tok = Tattribute ii} as id)
+    ::Parenthised (xxs,info_parens)
+    ::(PToken {tok = TPtVirg _} | PToken {tok = TEq _} | PToken {tok = TOBrace _})
+    ::xs
+     ->
+      pr2_cpp (Printf.sprintf "MACRO: attribute %s detected "
+		 (Ast_c.str_of_info ii));
+      id.tok <- TMacroGccEndAttr ii;
+      find_macro_paren xs
   | PToken ({tok = TattributeNoarg ii} as id)
     ::xs
      ->
@@ -1134,6 +1150,30 @@ let rec find_macro_paren xs =
 		 (Ast_c.str_of_info ii));
       TV.set_as_comment Token_c.CppAttr id;
       find_macro_paren xs
+  | PToken ({tok = TIdent _})::PToken ({tok = TIdent _})
+    ::PToken ({tok = TIdent (s,ii)} as id)
+    ::Parenthised (xxs,info_parens)
+    ::(PToken {tok = TPtVirg _} | PToken {tok = TEq _})
+    ::xs when (LP.current_context () = LP.InTopLevel &&
+	      s ==~ regexp_annot) ->
+      msg_attribute s;
+      id.tok <- TMacroEndAttrArgs (s,ii);
+      find_macro_paren xs
+  | PToken ({tok = TCCro _})::PToken ({tok = TIdent (s,ii)} as id)
+    ::Parenthised (xxs,info_parens)
+    ::(PToken {tok = TPtVirg _} | PToken {tok = TEq _})
+    ::xs when (LP.current_context () = LP.InTopLevel &&
+	      s ==~ regexp_annot) ->
+      msg_attribute s;
+      id.tok <- TMacroEndAttrArgs (s,ii);
+      find_macro_paren xs
+  | PToken ({tok = TMacroAttr (s,ii)} as attr)
+    ::Parenthised (xxs,info_parens)
+    ::(PToken {tok = TPtVirg _} | PToken {tok = TEq _} | PToken {tok = TOBrace _})
+    ::xs
+     ->
+      attr.tok <- TMacroEndAttrArgs (s,ii);
+      find_macro_paren xs
 
 (*
   (* attribute cpp, __xxx id *)
@@ -2064,6 +2104,15 @@ let lookahead2 ~pass next before =
 
 	(* christia *)
 
+	(* xx xx *)
+  | (TypedefIdent (s1, i1) :: _ , TypedefIdent (s2, i2) :: _) ->
+      TIdent (s1, i1)
+
+	(* xx * xx *)
+  | (TypedefIdent (s1, i1) :: _ , ptr)
+    when pointer ~followed_by:(function TypedefIdent _ -> true | _ -> false)  ptr  ->
+      TIdent (s1, i1)
+
 	(* delete[] *)
   | (TOCro i1 :: _, Tdelete _ :: _)
     when !Flag.c_plus_plus <> Flag.Off ->
@@ -2802,12 +2851,16 @@ let lookahead2 ~pass next before =
       when LP.current_context () = LP.InTopLevel &&
 	s1 ==~ regexp_annot ->
 	  msg_attribute s1;
-	  TMacroAttr (s1, i1)
+	  TMacroEndAttr (s1, i1)
 
-  | (TMacroAttr(s1,i1)::(TPtVirg(ii2)|TEq(ii2))::rest,_)
+  | (TMacroAttr(s1,i1)::(TPtVirg(ii2)|TEq(ii2)|TOBrace(ii2))::rest,_)
       ->
 	  TMacroEndAttr (s1, i1)
 
+  | (TMacroAttr(s1,i1)::TOPar(ii2)::t::rest,_)
+      when match t with TMul(_) -> false | _ -> true ->
+	  TMacroAttrArgs (s1, i1)
+
 (*  (* christia: here insert support for macros on top level *)
   | TIdent (s, ii) :: tl :: _, _ when
     can_be_on_top_level tl && LP.current_context () = InTopLevel ->
