@@ -27,7 +27,8 @@ let pr2, pr2_once = Common.mk_pr2_wrappers Flag_parsing_c.verbose_unparsing
 (* Types *)
 (*****************************************************************************)
 
-type type_with_ident = Ast_c.fullType -> (unit -> unit) -> unit
+type type_with_ident = Ast_c.fullType -> (unit -> unit) -> (unit -> unit) -> unit
+type type_with_ident_rest = Ast_c.fullType -> (unit -> unit) -> unit
 
 type 'a printer = 'a -> unit
 
@@ -50,12 +51,13 @@ type pretty_printers = {
   ty              : Ast_c.fullType printer;
   type_with_ident : type_with_ident;
   base_type       : Ast_c.fullType printer;
-  type_with_ident_rest : type_with_ident;
+  type_with_ident_rest : type_with_ident_rest;
   toplevel        : Ast_c.toplevel printer;
   fragment        : Ast_c.string_fragment printer;
   fragment_list   : (Ast_c.string_fragment list) printer;
   format          : Ast_c.string_format printer;
   attribute       : Ast_c.attribute printer;
+  attr_arg        : Ast_c.attr_arg printer;
   flow            : Control_flow_c.node printer;
   name            : Ast_c.name printer
 }
@@ -508,10 +510,11 @@ and pp_string_format (e,ii) =
 *)
   and (pp_type_with_ident:
 	 (unit -> unit) option -> (storage * il) option ->
-	   fullType -> attribute list -> attribute list ->
-	     unit) =
-    fun ident sto ft attrs endattrs ->
+	   fullType -> (unit -> unit) option -> attribute list ->
+	   attribute list ->  unit) =
+    fun ident sto ft midattr attrs endattrs ->
       pp_base_type ft  sto;
+      Common.do_option (fun f -> f()) midattr;
       (match (ident, Ast_c.unwrap_typeC ft) with
 	(Some _,_) | (_,Pointer _) -> pr_space()
       |	_ -> ());
@@ -691,8 +694,8 @@ and pp_string_format (e,ii) =
 		  | None -> None
                   | Some name -> Some (function _ -> pp_name name)
                 in
-		pp_type_with_ident identinfo None typ Ast_c.noattr
-		  Ast_c.noattr;
+		pp_type_with_ident identinfo None typ None
+		  Ast_c.noattr Ast_c.noattr;
 
 	    | (BitField (nameopt, typ, iidot, expr)), iivirg ->
                       (* first var cannot have a preceding ',' *)
@@ -702,7 +705,7 @@ and pp_string_format (e,ii) =
 		    pp_type typ;
 		| Some name ->
 		    pp_type_with_ident (Some (function _ -> pp_name name))
-		      None typ Ast_c.noattr Ast_c.noattr;
+		      None typ None Ast_c.noattr Ast_c.noattr;
 		    );
                 pr_elem iidot;
 		pp_expression expr
@@ -933,7 +936,7 @@ and pp_string_format (e,ii) =
         pp_type t
     | Some name ->
 	pp_type_with_ident (Some (function _ -> pp_name name))
-	  None t Ast_c.noattr Ast_c.noattr
+	  None t None Ast_c.noattr Ast_c.noattr
 
 
 
@@ -980,9 +983,9 @@ and pp_string_format (e,ii) =
     | (FunctionType _ | Array _ | Pointer _), _ -> raise (Impossible 111)
 
   and pp_type t =
-    pp_type_with_ident None None t Ast_c.noattr Ast_c.noattr
-  and pp_type_ident t i =
-    pp_type_with_ident (Some i) None t Ast_c.noattr Ast_c.noattr
+    pp_type_with_ident None None t None Ast_c.noattr Ast_c.noattr
+  and pp_type_ident t midattr i =
+    pp_type_with_ident (Some i) None t (Some midattr) Ast_c.noattr Ast_c.noattr
   and pp_type_ident_rest t i =
     pp_type_with_ident_rest (Some i) t Ast_c.noattr Ast_c.noattr
   and pp_base_type2 t =
@@ -994,6 +997,7 @@ and pp_string_format (e,ii) =
                    v_type = returnType;
                    v_storage = storage;
                    v_attr = attrs;
+                   v_midattr = midattrs;
                    v_endattr = endattrs;
                   },[])::xs),
 	       iivirg::ifakestart::iisto) ->
@@ -1008,7 +1012,7 @@ and pp_string_format (e,ii) =
 	| Some (name, iniopt) ->
 	    pp_type_with_ident
 	      (Some (function _ -> pp_name name)) (Some (storage, iisto))
-	      returnType attrs endattrs;
+	      returnType None  (attrs@midattrs) endattrs;
 	    (match iniopt with
 	      Ast_c.NoInit -> ()
 	    | Ast_c.ValInit(iini,init) ->
@@ -1025,6 +1029,7 @@ and pp_string_format (e,ii) =
 	    v_type = returnType;
 	    v_storage = storage2;
 	    v_attr = attrs;
+	    v_midattr = midattrs;
 	    v_endattr = endattrs;
 	  }, iivirg) ->
 
@@ -1149,8 +1154,22 @@ and pp_init (init, iinit) =
   and pp_attribute (e,ii) =
     match (e,ii) with
       Attribute(a), ii  ->
+        pp_attr_arg a
+    | GccAttribute(a), ii ->
+        let (i1,i2,i3,i4,i5) = Common.tuple_of_list5 ii in
+        pr_elem i1; pr_elem i2; pr_elem i3;
+        pp_attr_arg a; pr_elem i4; pr_elem i5;
+
+  and pp_attr_arg (e,ii) =
+    match (e,ii) with
+      MacroAttr(a), ii ->
         let (i) = Common.tuple_of_list1 ii in
         pr_elem i
+    | MacroAttrArgs(attr, args), ii ->
+        let (i1,i2,i3) = Common.tuple_of_list3 ii in
+        pr_elem i1; pr_elem i2;
+	pp_arg_list args;
+        pr_elem i3;
 
 (* ---------------------- *)
   and pp_def_start defbis iifunc1 iifunc2 ifakestart isto =
@@ -1163,7 +1182,7 @@ and pp_init (init, iinit) =
     pr_elem ifakestart;
 
     pp_type_with_ident None (Some (sto, isto))
-      returnt Ast_c.noattr Ast_c.noattr;
+      returnt None Ast_c.noattr Ast_c.noattr;
 
     pp_attributes pr_elem pr_space attrs;
     pr_space();
@@ -1532,6 +1551,7 @@ and pp_init (init, iinit) =
     fragment   = pp_string_fragment;
     fragment_list = pp_string_fragment_list;
     attribute  = pp_attribute;
+    attr_arg   = pp_attr_arg;
     format     = pp_string_format;
     flow       = pp_flow;
     name       = pp_name;
@@ -1592,6 +1612,7 @@ let pp_toplevel_simple   = ppc.toplevel
 let pp_string_fragment_simple = ppc.fragment
 let pp_string_format_simple = ppc.format
 let pp_attribute_simple  = ppc.attribute
+let pp_attr_arg_simple   = ppc.attr_arg
 let pp_flow_simple       = ppc.flow
 let pp_name              = ppc.name
 
@@ -1661,6 +1682,9 @@ let pp_string_format_gen ~pr_elem ~pr_space =
 let pp_attribute_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).attribute
 
+let pp_attr_arg_gen ~pr_elem ~pr_space =
+  (pp_elem_sp pr_elem pr_space).attr_arg
+
 let pp_program_gen ~pr_elem ~pr_space =
   (pp_elem_sp pr_elem pr_space).toplevel
 
