@@ -694,8 +694,12 @@ and fullType ft =
     Ast.Type(_,cv,ty) ->
       (match Ast.unwrap ty with
 	Ast.Pointer(_,_) ->
-	  typeC ty; print_option_prespace (mcode const_vol) cv
-      |	_ -> print_option_space (mcode const_vol) cv; typeC ty)
+          typeC ty;
+          param_print_between (function _ -> ()) (fun c -> pr_space();
+          mcode const_vol c) cv
+      | _ ->
+          param_print_between (function _ -> pr_space()) (mcode const_vol) cv;
+          typeC ty)
 
   | Ast.AsType(ty, asty) -> fullType ty
   | Ast.DisjType _ | Ast.ConjType _ -> raise CantBeInPlus
@@ -707,19 +711,34 @@ and print_fninfo = function
   | Ast.FInline(inline) -> mcode print_string inline; pr_space()
   | Ast.FAttr(attr) -> print_attribute attr; pr_space()
 
-and print_attribute_list attrs =
-  if not (attrs = []) then pr_space();
-  print_between pr_space print_attribute attrs
+and print_attribute_list ?(befspace=true) ?(aftspace=false) attrs =
+  if befspace && not (attrs = []) then pr_space();
+  print_between pr_space print_attribute attrs;
+  if aftspace && not (attrs = []) then pr_space()
 
 and print_attribute attr =
   match Ast.unwrap attr with
-    Ast.Attribute(a) -> mcode print_string a
-  | Ast.MetaAttribute(name,_,_,_) ->
+    Ast.Attribute(a) -> print_attr_arg a
+  | Ast.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+      mcode print_string attr_;
+      mcode print_string_box lp1; mcode print_string_box lp2;
+      print_attr_arg arg; close_box();
+      mcode print_string_box rp1; close_box();
+      mcode print_string_box rp2;
+
+and print_attr_arg arg =
+  match Ast.unwrap arg with
+    Ast.MacroAttr(arg) -> mcode print_string arg
+  | Ast.MacroAttrArgs(attr,lp,args,rp) ->
+      mcode print_string attr; mcode (print_string_with_hint StartBox) lp;
+      dots (function _ -> ()) arg_expression args;
+      mcode (print_string_with_hint EndBox) rp
+  | Ast.MetaAttr(name,_,_,_) ->
       handle_metavar name
 	(function
-	    Ast_c.MetaAttributeVal a ->
-              pretty_print_c.Pretty_print_c.attribute a
-          | _ -> error name attr "attribute value expected")
+	    Ast_c.MetaAttrArgVal a ->
+              pretty_print_c.Pretty_print_c.attr_arg a
+          | _ -> error name arg "attr_arg value expected")
 
 and typeC ty =
   match Ast.unwrap ty with
@@ -729,7 +748,7 @@ and typeC ty =
   | Ast.Pointer(ty,star) ->
       fullType ty; ft_space ty; mcode print_string star; eatspace()
   | Ast.ParenType(lp,ty,rp) ->
-      print_parentype (lp,ty,rp) (function _ -> ())
+      print_parentype (lp,[],ty,rp) (function _ -> ())
   | Ast.FunctionType(ty,lp,params,rp) ->
       fullType ty;
       mcode print_string lp;
@@ -803,7 +822,7 @@ and storage = function
 (* --------------------------------------------------------------------- *)
 (* ParenType *)
 
-and print_parentype (lp,ty,rp) fn =
+and print_parentype (lp,midattr,ty,rp) fn =
   let function_pointer ty1 array_dec =
     match Ast.unwrap ty1 with
      Ast.Type(_,_,fty1) ->
@@ -814,7 +833,7 @@ and print_parentype (lp,ty,rp) fn =
             (match Ast.unwrap fty3 with
               Ast.FunctionType(ty3,lp3,params,rp3) ->
                fullType ty3;
-               pr_space();
+               print_attribute_list midattr ~befspace:true ~aftspace:true;
                mcode print_string lp;
                mcode print_string star;
                fn();
@@ -846,9 +865,9 @@ and print_parentype (lp,ty,rp) fn =
 (* --------------------------------------------------------------------- *)
 (* Variable declaration *)
 
-and print_named_type ty id =
+and print_named_type ty midattr id =
   match Ast.unwrap ty with
-    Ast.Type(_,None,ty1) ->
+    Ast.Type(_,[],ty1) ->
       (match Ast.unwrap ty1 with
         Ast.Array(_,_,_,_) ->
 	  let rec loop ty k =
@@ -856,7 +875,7 @@ and print_named_type ty id =
 	      Ast.Array(ty,lb,size,rb) ->
 		(match Ast.unwrap ty with
 		  Ast.Type(_,cv,ty) ->
-		    print_option_space (mcode const_vol) cv;
+		    param_print_between (function _ -> pr_space()) (mcode const_vol) cv;
 		    loop ty
 		      (function _ ->
 			k ();
@@ -864,17 +883,45 @@ and print_named_type ty id =
 			print_option expression size;
 			mcode print_string rb)
 		| _ -> failwith "complex array types not supported")
-	    | _ -> typeC ty; ty_space ty; id(); k () in
+	    | _ -> typeC ty; ty_space ty; print_attribute_list midattr ~befspace:false ~aftspace:true; id(); k () in
 	  loop ty1 (function _ -> ())
       | Ast.MetaType(name,_,_,_) ->
-	  handle_metavar name  (function
+          (* MetaType with an array and a pointer have to be treated specially.
+           * Example(Coccinelle):
+           *  T
+           *  +__attribute__((attr))
+           *  p;
+           *
+           * Example(C):
+           *  int array[41];
+           *
+           * should be printed:
+           * int __attribute__((attr)) array[41];
+           *
+           * However, without this secial care, __attribute__((attr)) is put
+           * just after the last character of T (']' in this case) like:
+           * int array[41]__attribute__((attr));
+           *)
+          let (a,info,b,c) = name in
+          let (res,name_string,line,lcol,rcol) = lookup_metavar name in
+          let (info_new,straft) = match res with
+              Some (Ast_c.MetaTypeVal ty) ->
+                let (qu, iiqu), (tyy, iity) = ty in
+                (match tyy with
+                    Ast_c.Pointer _
+                  | Ast_c.Array _ -> {info with Ast.straft = [] },info.Ast.straft
+                  | _ -> info,[])
+            | _ -> error name ty "type value expected" in
+          handle_metavar (a,info_new,b,c) (function
               Ast_c.MetaTypeVal ty ->
 		pretty_print_c.Pretty_print_c.type_with_ident ty
+                  (function _ -> print_attribute_list midattr ~befspace:false ~aftspace:false)
 		  (function _ -> id())
             | _ -> error name ty "type value expected")
       | Ast.ParenType(lp,ty,rp) ->
-          print_parentype (lp,ty,rp) (function _ -> id())
+          print_parentype (lp,midattr,ty,rp) (function _ -> id())
       | Ast.FunctionType(ty,lp,params,rp) ->
+          assert (midattr = []);
           fullType ty;
           id();
           mcode print_string lp;
@@ -883,8 +930,8 @@ and print_named_type ty id =
     (*| should have a case here for pointer to array or function type
         that would put ( * ) around the variable.  This makes one wonder
         why we really need a special case for function pointer *)
-      | _ -> fullType ty; ft_space ty; id())
-  | _ -> fullType ty; ft_space ty; id()
+      | _ -> fullType ty; ft_space ty; print_attribute_list midattr ~befspace:false ~aftspace:true; id())
+  | _ -> fullType ty; ft_space ty; print_attribute_list midattr ~befspace:false ~aftspace:true; id()
 
 and ty_space ty =
   match Ast.unwrap ty with
@@ -924,21 +971,22 @@ and declaration d =
 
   | Ast.AsDecl(decl,asdecl) -> declaration decl
 
-  | Ast.Init(stg,ty,id,attr,eq,ini,sem) ->
+  | Ast.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
       print_option (mcode storage) stg;
       print_option (function _ -> pr_space()) stg;
-      print_named_type ty (fun _ -> ident id);
-      print_attribute_list attr;
+      print_named_type ty midattr (fun _ -> ident id);
+      print_attribute_list endattr;
       pr_space(); mcode print_string eq;
       pr_space(); initialiser true ini; mcode print_string sem
-  | Ast.UnInit(stg,ty,id,attr,sem) ->
+  | Ast.UnInit(stg,ty,midattr,id,endattr,sem) ->
       print_option (mcode storage) stg;
       print_option (function _ -> pr_space()) stg;
-      print_named_type ty (fun _ -> ident id);
-      print_attribute_list attr;
+      print_named_type ty midattr (fun _ -> ident id);
+      print_attribute_list endattr;
       mcode print_string sem
-  | Ast.FunProto (fninfo,name,lp1,params,va,rp1,sem) ->
+  | Ast.FunProto (fninfo,attr,name,lp1,params,va,rp1,sem) ->
       List.iter print_fninfo fninfo;
+      print_attribute_list attr ~befspace:false ~aftspace:true;
       ident name; mcode print_string_box lp1;
       parameter_list params;
       begin match va with
@@ -971,7 +1019,7 @@ and declaration d =
       mcode print_string sem
   | Ast.Typedef(stg,ty,id,sem) ->
       mcode print_string stg; pr_space();
-      print_named_type ty (fun _ -> typeC id);
+      print_named_type ty [] (fun _ -> typeC id);
       mcode print_string sem
   | Ast.DisjDecl(_) | Ast.ConjDecl(_) -> raise CantBeInPlus
   | Ast.OptDecl(decl) -> raise CantBeInPlus
@@ -1002,7 +1050,7 @@ and field d =
       begin
 	match id with
 	  None -> fullType ty
-	| Some id -> print_named_type ty (fun _ -> ident id)
+        | Some id -> print_named_type ty [] (fun _ -> ident id)
       end;
       let bitfield (c, e) =
 	mcode print_string c;
@@ -1110,11 +1158,12 @@ and parameterTypeDef p =
     Ast.VoidParam(ty,attr) ->
       fullType ty;
       print_attribute_list attr;
-  | Ast.Param(ty,Some id,attr) ->
-      print_named_type ty (fun _ -> ident id);
+  | Ast.Param(ty,midattr,Some id,attr) ->
+      print_named_type ty midattr (fun _ -> ident id);
       print_attribute_list attr;
-  | Ast.Param(ty,None,attr) ->
+  | Ast.Param(ty,midattr,None,attr) ->
       fullType ty;
+      assert (midattr = []);
       print_attribute_list attr;
   | Ast.MetaParam(name,_,_,_) ->
       handle_metavar name
@@ -1161,7 +1210,7 @@ and inc_elem = function
 
 and rule_elem arity re =
   match Ast.unwrap re with
-    Ast.FunHeader(_,_,fninfo,name,lp,params,va,rp) ->
+    Ast.FunHeader(_,_,fninfo,name,lp,params,va,rp,attrs) ->
       pr_arity arity; List.iter print_fninfo fninfo;
       ident name; mcode print_string_box lp;
       parameter_list params;
@@ -1172,6 +1221,7 @@ and rule_elem arity re =
           mcode print_string ellipsis
       end;
       close_box(); mcode print_string rp;
+      print_attribute_list attrs;
       force_newline()
   | Ast.Decl decl -> pr_arity arity; annotated_decl decl
 
@@ -1519,6 +1569,7 @@ let pp_any = function
   | Ast.CaseLineTag(x) -> case_line "" x; false
   | Ast.StringFragmentTag(x) -> string_fragment x; false
   | Ast.AttributeTag(x) -> print_attribute x; false
+  | Ast.AttrArgTag(x) -> print_attr_arg x; false
 
   | Ast.ConstVolTag(x) -> const_vol x unknown unknown; false
   | Ast.Directive(xs) ->
