@@ -137,7 +137,7 @@ let get_function_name rule env =
       donothing expression donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing
-      donothing donothing donothing donothing donothing).V.combiner_top_level
+      donothing donothing donothing donothing donothing donothing).V.combiner_top_level
       rule in
   match names with
     [name] ->
@@ -209,6 +209,8 @@ let print_metavar pr = function
          p_namei = Some name';
          p_type = (({Ast_c.const = false; Ast_c.volatile = false},[]),ty);
          p_attr = [];
+         p_midattr = [];
+         p_endattr = [];
         }
   | _ -> failwith "function must have named parameters"
 
@@ -251,7 +253,7 @@ let print_extra_typedefs pr env =
       | Ast_c.MetaFieldListVal(fields) ->
 	  Visitor_c.vk_struct_fields bigf fields
       | Ast_c.MetaFmtVal(fmt) -> Visitor_c.vk_string_format bigf fmt
-      | Ast_c.MetaAttributeVal(attr) -> Visitor_c.vk_attribute bigf attr
+      | Ast_c.MetaAttrArgVal(name) -> Visitor_c.vk_attr_arg bigf name
       | Ast_c.MetaFragListVal(frags) ->
 	  Visitor_c.vk_string_fragments bigf frags
       | Ast_c.MetaStmtVal(stm,_,_) -> Visitor_c.vk_statement bigf stm
@@ -314,8 +316,8 @@ let rename argids env =
 	   Ast_c.MetaFieldListVal(Visitor_c.vk_struct_fields_s bigf stm)
        | Ast_c.MetaFmtVal(fmt) ->
 	   Ast_c.MetaFmtVal(Visitor_c.vk_string_format_s bigf fmt)
-       | Ast_c.MetaAttributeVal(attr) ->
-	   Ast_c.MetaAttributeVal(Visitor_c.vk_attribute_s bigf attr)
+       | Ast_c.MetaAttrArgVal(name) ->
+	   Ast_c.MetaAttrArgVal(Visitor_c.vk_attr_arg_s bigf name)
        | Ast_c.MetaFragListVal(frags) ->
 	   Ast_c.MetaFragListVal(Visitor_c.vk_string_fragments_s bigf frags)
        | Ast_c.MetaStmtVal(stm,original,ty) ->
