@@ -324,7 +324,10 @@ let rec vk_expr = fun bigf expr ->
 
     | SizeOfExpr  (e) -> exprf e
     | SizeOfType  (t) -> vk_type bigf t
-    | Cast    (t, e) -> vk_type bigf t; exprf e
+    | Cast    (t, a, e) ->
+        vk_type bigf t;
+        a +> List.iter (vk_attribute bigf);
+        exprf e
 
     (* old: | StatementExpr (((declxs, statxs), is)), is2 ->
      *          List.iter (vk_decl bigf) declxs;
@@ -516,6 +519,7 @@ and vk_type = fun bigf t ->
     | ParenType t -> typef t
     | TypeOfExpr e -> vk_expr bigf e
     | TypeOfType t -> typef t
+    | AutoType -> ()
 
   in typef t
 
@@ -523,9 +527,15 @@ and vk_type = fun bigf t ->
 and vk_attribute = fun bigf attr ->
   let iif ii = vk_ii bigf ii in
   match attr with
-  | Attribute s, ii ->
-      iif ii
+  | Attribute s, ii -> iif ii; vk_attr_arg bigf s
+  | GccAttribute s, ii -> iif ii; vk_attr_arg bigf s
 
+and vk_attr_arg = fun bigf name ->
+  let iif ii = vk_ii bigf ii in
+  match name with
+  | MacroAttr s, ii -> iif ii
+  | MacroAttrArgs (attr, args), ii ->
+      vk_argument_list bigf args; iif ii
 
 and vk_exec_code = fun bigf e ->
   let iif ii = vk_ii bigf ii in
@@ -547,8 +557,9 @@ and vk_decl = fun bigf d ->
         iif ii;
         vk_onedecl bigf x
       );
-    | MacroDecl ((_stob, s, args, ptvg),ii) ->
+    | MacroDecl ((_stob, s, args, attrs, ptvg),ii) ->
         iif ii;
+        attrs +> List.iter (vk_attribute bigf);
         vk_argument_list bigf args
     | MacroDeclInit ((_stob, s, args, ini),ii) ->
         iif ii;
@@ -570,6 +581,7 @@ and vk_onedecl_opt process_type = fun bigf onedecl ->
       v_type_bis = tbis;
       v_storage = _sto;
       v_attr = attrs;
+      v_midattr = midattrs;
       v_endattr = endattrs})  ->
 
     (if process_type then vk_type bigf t);
@@ -703,12 +715,14 @@ and vk_def = fun bigf d ->
        f_storage = sto;
        f_body = statxs;
        f_attr = attrs;
+       f_endattr = endattrs;
        f_old_c_style = oldstyle;
       }, ii
         ->
         iif ii;
         iif iib;
         attrs +> List.iter (vk_attribute bigf);
+        endattrs +> List.iter (vk_attribute bigf);
         vk_type bigf returnt;
         vk_name bigf name;
         paramst +> List.iter (fun (param,iicomma) ->
@@ -1187,7 +1201,11 @@ let rec vk_expr_s = fun bigf expr ->
 
       | SizeOfExpr  (e) -> SizeOfExpr (exprf e)
       | SizeOfType  (t) -> SizeOfType (vk_type_s bigf t)
-      | Cast    (t, e)  -> Cast (vk_type_s bigf t, exprf e)
+      | Cast    (t, a, e)  ->
+          Cast
+            (vk_type_s bigf t,
+             a +> List.map (vk_attribute_s bigf),
+             exprf e)
 
       | StatementExpr (statxs, is) ->
           StatementExpr (
@@ -1433,6 +1451,7 @@ and vk_type_s = fun bigf t ->
       | ParenType t -> ParenType (typef t)
       | TypeOfExpr e -> TypeOfExpr (vk_expr_s bigf e)
       | TypeOfType t -> TypeOfType (typef t)
+      | AutoType -> AutoType
     in
     (q', iif_iiq),
     (t', iif iit)
@@ -1444,7 +1463,19 @@ and vk_attribute_s = fun bigf attr ->
   let iif ii = vk_ii_s bigf ii in
   match attr with
   | Attribute s, ii ->
-      Attribute s, iif ii
+      Attribute (vk_attr_arg_s bigf s), iif ii
+  | GccAttribute s, ii ->
+      GccAttribute (vk_attr_arg_s bigf s), iif ii
+
+and vk_attr_arg_s = fun bigf name ->
+  let iif ii = vk_ii_s bigf ii in
+  match name with
+  | MacroAttr s, ii -> MacroAttr s, iif ii
+  | MacroAttrArgs (attr, args), ii ->
+      MacroAttrArgs (attr,
+        args +> List.map (fun (e,ii) ->
+          vk_argument_s bigf e, iif ii
+        )), iif ii
 
 and vk_exec_code_s = fun bigf e ->
   let iif ii = vk_ii_s bigf ii in
@@ -1462,10 +1493,11 @@ and vk_decl_s = fun bigf d ->
     | DeclList (xs, ii) ->
         DeclList (List.map (fun (x,ii) -> (vk_onedecl_s bigf x, iif ii)) xs,
 		  iif ii)
-    | MacroDecl ((stob, s, args, ptvg),ii) ->
+    | MacroDecl ((stob, s, args, attrs, ptvg),ii) ->
         MacroDecl
           ((stob, s,
            args +> List.map (fun (e,ii) -> vk_argument_s bigf e, iif ii),
+           attrs +> List.map (vk_attribute_s bigf),
            ptvg),
           iif ii)
     | MacroDeclInit ((stob, s, args, ini),ii) ->
@@ -1483,6 +1515,7 @@ and vk_onedecl_opt_s process_type bigf {v_namei = var;
             v_storage = sto;
             v_local= local;
             v_attr = attrs;
+            v_midattr = midattrs;
             v_endattr = endattrs} =
   let iif ii = vk_ii_s bigf ii in
     {v_namei =
@@ -1503,6 +1536,7 @@ and vk_onedecl_opt_s process_type bigf {v_namei = var;
      v_storage = sto;
      v_local = local;
      v_attr = attrs +> List.map (vk_attribute_s bigf);
+     v_midattr = midattrs +> List.map (vk_attribute_s bigf);
      v_endattr = endattrs +> List.map (vk_attribute_s bigf);
     }
 
@@ -1614,6 +1648,7 @@ and vk_def_s = fun bigf d ->
        f_storage = sto;
        f_body = statxs;
        f_attr = attrs;
+       f_endattr = endattrs;
        f_old_c_style = oldstyle;
       }, ii
         ->
@@ -1628,6 +1663,8 @@ and vk_def_s = fun bigf d ->
             vk_statement_sequencable_list_s bigf statxs;
          f_attr =
             attrs +> List.map (vk_attribute_s bigf);
+         f_endattr =
+            endattrs +> List.map (vk_attribute_s bigf);
          f_old_c_style =
             oldstyle +> Common.map_option (fun decls ->
               decls +> List.map (vk_decl_s bigf)
@@ -1662,7 +1699,7 @@ and vk_toplevel_s = fun bigf p ->
   in f (k, bigf) p
 
 and vk_program_s : visitor_c_s -> toplevel list -> toplevel list =
-      fun bigf -> List.map (vk_toplevel_s bigf)
+      fun bigf ast -> List.rev (List.rev_map (vk_toplevel_s bigf) ast)
 
 
 and vk_cpp_directive_s = fun bigf top ->
@@ -1789,7 +1826,7 @@ and vk_info_s = fun bigf info ->
   infof info
 
 and vk_ii_s = fun bigf ii ->
-  List.map (vk_info_s bigf) ii
+  List.rev (List.rev_map (vk_info_s bigf) ii)
 
 (* ------------------------------------------------------------------------ *)
 
@@ -1927,10 +1964,19 @@ and vk_node_s = fun bigf node ->
 (* ------------------------------------------------------------------------ *)
 and vk_param_s = fun bigf param ->
   let iif ii = vk_ii_s bigf ii in
-  let {p_namei = swrapopt; p_register = (b, iib); p_type=ft} = param in
+  let
+    {p_namei = swrapopt;
+     p_register = (b, iib);
+     p_type=ft;
+     p_attr = attrs;
+     p_midattr = midattrs;
+     p_endattr = endattrs} = param in
   { p_namei = swrapopt +> Common.map_option (vk_name_s bigf);
     p_register = (b, iif iib);
     p_type = vk_type_s bigf ft;
+    p_attr = attrs +> List.map (vk_attribute_s bigf);
+    p_midattr = midattrs +> List.map (vk_attribute_s bigf);
+    p_endattr = endattrs +> List.map (vk_attribute_s bigf);
   }
 
 let vk_arguments_s = fun bigf args ->
