@@ -87,6 +87,7 @@ val vk_ident_list_splitted : visitor_c -> (name, il) Common.either list -> unit
 val vk_exec_code_list_splitted :
     visitor_c -> (exec_code, il) Common.either list -> unit
 val vk_attribute       : visitor_c -> attribute -> unit
+val vk_attr_arg       : visitor_c -> attr_arg -> unit
 val vk_attrs_splitted :
     visitor_c -> (attribute, il) Common.either list -> unit
 
@@ -208,6 +209,7 @@ val vk_exec_code_list_splitted_s :
 	(exec_code, il) Common.either list
 
 val vk_attribute_s : visitor_c_s -> attribute -> attribute
+val vk_attr_arg_s : visitor_c_s -> attr_arg -> attr_arg
 val vk_attrs_splitted_s :
     visitor_c_s ->
       (attribute, il) Common.either list ->
