@@ -390,9 +390,9 @@ and make_typeC =
 and top_typeC tgt opt_allowed typ =
   match Ast0.unwrap typ with
     Ast0.ConstVol(cv,ty) ->
-      let arity = all_same opt_allowed tgt (mcode2line cv)
-	  [mcode2arity cv] in
-      let cv = mcode cv in
+      let arity = all_same opt_allowed tgt (mcode2line (List.hd cv))
+	  (List.map mcode2arity cv) in
+      let cv = List.map mcode cv in
       let ty = typeC arity ty in
       make_typeC typ tgt arity (Ast0.ConstVol(cv,ty))
   | Ast0.BaseType(ty,strings) ->
@@ -556,31 +556,33 @@ and declaration tgt decl =
       let arity = all_same true tgt (mcode2line name) [mcode2arity name] in
       let name = mcode name in
       make_decl decl tgt arity (Ast0.MetaDecl(name,cstr,pure))
-  | Ast0.Init(stg,ty,id,attr,eq,exp,sem) ->
+  | Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem) ->
       let arity =
 	all_same true tgt (mcode2line eq)
 	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
 	   (List.map mcode2arity [eq;sem])) in
       let stg = get_option mcode stg in
       let ty = typeC arity ty in
+      let midattr = List.map (attribute arity) midattr in
       let id = ident false arity id in
-      let attr = List.map (attribute arity) attr in
+      let endattr = List.map (attribute arity) endattr in
       let eq = mcode eq in
       let exp = initialiser arity exp in
       let sem = mcode sem in
-      make_decl decl tgt arity (Ast0.Init(stg,ty,id,attr,eq,exp,sem))
-  | Ast0.UnInit(stg,ty,id,attr,sem) ->
+      make_decl decl tgt arity (Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem))
+  | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
       let arity =
 	all_same true tgt (mcode2line sem)
 	  ((match stg with None -> [] | Some x -> [mcode2arity x]) @
 	   [mcode2arity sem]) in
       let stg = get_option mcode stg in
       let ty = typeC arity ty in
+      let midattr = List.map (attribute arity) midattr in
       let id = ident false arity id in
-      let attr = List.map (attribute arity) attr in
+      let endattr = List.map (attribute arity) endattr in
       let sem = mcode sem in
-      make_decl decl tgt arity (Ast0.UnInit(stg,ty,id,attr,sem))
-  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) ->
+      make_decl decl tgt arity (Ast0.UnInit(stg,ty,midattr,id,endattr,sem))
+  | Ast0.FunProto(fi,attr,name,lp1,params,va,rp1,sem) ->
     let tokens = match va with
       | None -> [lp1;rp1;sem]
       | Some (c1,e1) -> [lp1;c1;e1;rp1;sem] in
@@ -588,6 +590,7 @@ and declaration tgt decl =
 	all_same true tgt (mcode2line lp1)
 	  (List.map mcode2arity tokens) in
       let fi = List.map (fninfo arity) fi in
+      let attr = List.map (attribute arity) attr in
       let name = ident false arity name in
       let lp1 = mcode lp1 in
       let params = parameter_list tgt params in
@@ -596,7 +599,7 @@ and declaration tgt decl =
         | Some (c1, e1) -> Some (mcode c1, mcode e1) in
       let rp1 = mcode rp1 in
       let sem = mcode sem in
-      make_decl decl tgt arity (Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
+      make_decl decl tgt arity (Ast0.FunProto(fi,attr,name,lp1,params,va,rp1,sem))
   | Ast0.MacroDecl(stg,name,lp,args,rp,attr,sem) ->
       let arity =
 	all_same true tgt (mcode2line lp)
@@ -820,27 +823,29 @@ and parameterTypeDef tgt param =
     Ast0.VoidParam(ty,attr) ->
       Ast0.rewrap param
         (Ast0.VoidParam(typeC tgt ty,List.map (attribute tgt) attr))
-  | Ast0.Param(ty,Some id,attr) ->
+  | Ast0.Param(ty,midattr,Some id,attr) ->
       let ty = top_typeC tgt true ty in
+      let midattr = List.map (attribute tgt) midattr in
       let id = ident true tgt id in
       let attr = List.map (attribute tgt) attr in
       Ast0.rewrap param
 	(match (Ast0.unwrap ty,Ast0.unwrap id) with
 	  (Ast0.OptType(ty),Ast0.OptIdent(id)) ->
-	    Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,Some id,attr)))
+	    Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,midattr,Some id,attr)))
 	| (Ast0.OptType(ty),_) ->
 	    fail param "arity mismatch in param declaration"
 	| (_,Ast0.OptIdent(id)) ->
 	    fail param "arity mismatch in param declaration"
-	| _ -> Ast0.Param(ty,Some id,attr))
-  | Ast0.Param(ty,None,attr) ->
+	| _ -> Ast0.Param(ty,midattr,Some id,attr))
+  | Ast0.Param(ty,midattr,None,attr) ->
       let ty = top_typeC tgt true ty in
       let attr = List.map (attribute tgt) attr in
+      assert (midattr = []);
       Ast0.rewrap param
 	(match Ast0.unwrap ty with
 	  Ast0.OptType(ty) ->
-	    Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,None,attr)))
-	| _ -> Ast0.Param(ty,None,attr))
+            Ast0.OptParam(Ast0.rewrap param (Ast0.Param(ty,[],None,attr)))
+        | _ -> Ast0.Param(ty,[],None,attr))
   | Ast0.MetaParam(name,cstr,pure) ->
       let arity = param_same (mcode2line name) [mcode2arity name] in
       let name = mcode name in
@@ -1152,7 +1157,7 @@ and statement tgt stm =
 	     (expression Ast0.NONE))
 	  whn in
       make_rule_elem stm tgt arity (Ast0.Dots(d,whn))
-  | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
+  | Ast0.FunDecl(bef,fi,name,lp,params,va,rp,attrs,lbrace,body,rbrace,aft) ->
       let arity =
 	all_same true tgt (mcode2line lp)
 	  ((List.map mcode2arity [lp;rp;lbrace;rbrace]) @ (fninfo2arity fi)) in
@@ -1164,11 +1169,12 @@ and statement tgt stm =
         | None -> None
         | Some (comma, ellipsis) -> Some (mcode comma, mcode ellipsis) in
       let rp = mcode rp in
+      let attrs = List.map (attribute tgt) attrs in
       let lbrace = mcode lbrace in
       let body = dots (statement arity) body in
       let rbrace = mcode rbrace in
       make_rule_elem stm tgt arity
-	(Ast0.FunDecl(bef,fi,name,lp,params,newva,rp,lbrace,body,rbrace,aft))
+	(Ast0.FunDecl(bef,fi,name,lp,params,newva,rp,attrs,lbrace,body,rbrace,aft))
   | Ast0.Include(inc,s) ->
       let arity =
 	all_same true tgt (mcode2line inc) [mcode2arity inc; mcode2arity s] in
@@ -1281,12 +1287,37 @@ and make_attribute =
 
 and attribute tgt attr =
   match Ast0.unwrap attr with
-    Ast0.Attribute(a) ->
-      Ast0.rewrap attr (Ast0.Attribute(mcode a))
-  | Ast0.MetaAttribute(name,cstr,pure) ->
+    Ast0.Attribute(arg) ->
+      let arg = attr_arg tgt arg in
+      Ast0.rewrap attr (Ast0.Attribute(arg))
+  | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+      let arity = all_same false tgt (mcode2line lp1) [mcode2arity lp1;mcode2arity rp2] in
+      let attr_ = mcode attr_ in
+      let lp1 = mcode lp1 in
+      let lp2 = mcode lp2 in
+      let arg = attr_arg tgt arg in
+      let rp1 = mcode rp1 in
+      let rp2 = mcode rp2 in
+      make_attribute attr tgt arity (Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2))
+
+and make_attr_arg =
+  make_opt
+    (function x -> failwith "opt not allowed for attr_arg")
+
+and attr_arg tgt arg =
+  match Ast0.unwrap arg with
+    Ast0.MacroAttr(name) -> Ast0.rewrap arg (Ast0.MacroAttr(mcode name))
+  | Ast0.MetaAttr(name,cstr,pure) ->
       let arity = all_same false tgt (mcode2line name) [mcode2arity name] in
       let name = mcode name in
-      make_attribute attr tgt arity (Ast0.MetaAttribute(name,cstr,pure))
+      make_attr_arg arg tgt arity (Ast0.MetaAttr(name,cstr,pure))
+  | Ast0.MacroAttrArgs(attr,lp,args,rp) ->
+      let arity = all_same false tgt (mcode2line lp) [mcode2arity lp;mcode2arity rp] in
+      let attr = mcode attr in
+      let lp = mcode lp in
+      let args = dots (expression arity) args in
+      let rp = mcode rp in
+      make_attr_arg arg tgt arity (Ast0.MacroAttrArgs(attr,lp,args,rp))
 
 and whencode notfn alwaysfn expression = function
     Ast0.WhenNot (w,e,a) -> Ast0.WhenNot (w,e,notfn a)
