@@ -187,7 +187,7 @@ and binaryOp = base_binaryOp wrap
 (* Types *)
 
 and base_typeC =
-    ConstVol        of Ast_cocci.const_vol mcode * typeC
+    ConstVol        of Ast_cocci.const_vol mcode list * typeC
   | BaseType        of Ast_cocci.baseType * string mcode list
   | Signed          of Ast_cocci.sign mcode * typeC option
   | Pointer         of typeC * string mcode (* * *)
@@ -231,12 +231,12 @@ and typeC = base_typeC wrap
 and base_declaration =
     MetaDecl   of Ast_cocci.meta_name mcode * constraints * pure (* variables *)
   | AsDecl        of declaration * declaration
-  | Init       of Ast_cocci.storage mcode option * typeC * ident * attr list *
-	string mcode (*=*) * initialiser * string mcode (*;*)
-  | UnInit     of Ast_cocci.storage mcode option * typeC * ident * attr list *
-	string mcode (* ; *)
+  | Init of Ast_cocci.storage mcode option * typeC * attr list * ident *
+        attr list * string mcode (*=*) * initialiser * string mcode (*;*)
+  | UnInit of Ast_cocci.storage mcode option * typeC * attr list * ident *
+	attr list * string mcode (* ; *)
   | FunProto of
-	fninfo list * ident (* name *) *
+	fninfo list * attr list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
         (string mcode (* , *) * string mcode (* ...... *) ) option *
 	string mcode (* ) *) * string mcode (* ; *)
@@ -322,7 +322,7 @@ and initialiser_list = initialiser dots
 
 and base_parameterTypeDef =
     VoidParam     of typeC * attr list
-  | Param         of typeC * ident option * attr list
+  | Param         of typeC * attr list * ident option * attr list
   | MetaParam     of Ast_cocci.meta_name mcode * constraints * pure
   | MetaParamList of Ast_cocci.meta_name mcode * listlen * constraints * pure
   | AsParam       of parameterTypeDef * expression (* expr, always metavar *)
@@ -415,7 +415,7 @@ and base_statement =
 	fninfo list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
 	(string mcode (* , *) * string mcode (* ...... *) ) option *
-	string mcode (* ) *) *
+	string mcode (* ) *) * attr list *
 	string mcode (* { *) * statement dots *
 	string mcode (* } *) *
 	(info * mcodekind) (* after the function decl *)
@@ -446,11 +446,21 @@ and fninfo =
   | FAttr of attr
 
 and base_attr =
-    Attribute of string mcode
-  | MetaAttribute of Ast_cocci.meta_name mcode * constraints * pure
+    Attribute of attr_arg
+  | GccAttribute of string mcode (* __attribute__ *) *
+                    string mcode (* ( *) * string mcode (* ( *) *
+                    attr_arg * string mcode (* ) *) * string mcode (* ) *)
 
 and attr = base_attr wrap
 
+and base_attr_arg =
+    MacroAttr of string mcode
+  | MacroAttrArgs of string mcode * string mcode (* ( *) *
+                      expression dots * string mcode (* ) *)
+  | MetaAttr of Ast_cocci.meta_name mcode * constraints * pure
+
+and attr_arg = base_attr_arg wrap
+
 and ('a,'b) whencode =
     WhenNot of string mcode (* when *) * string mcode (* != *) * 'a
   | WhenAlways of string mcode (* when *) * string mcode (* = *) * 'b
@@ -568,6 +578,7 @@ and anything =
   | CaseLineTag of case_line
   | StringFragmentTag of string_fragment
   | AttributeTag of attr
+  | AttrArgTag of attr_arg
   | TopTag of top_level
   | IsoWhenTag of Ast_cocci.when_modifier (*only for when code, in iso phase*)
   | IsoWhenTTag of expression(*only for when code, in iso phase*)
@@ -601,6 +612,7 @@ val forinfo : forinfo -> anything
 val case_line : case_line -> anything
 val string_fragment : string_fragment -> anything
 val attr : attr -> anything
+val attr_arg : attr_arg -> anything
 val top : top_level -> anything
 
 (* --------------------------------------------------------------------- *)
