@@ -190,7 +190,7 @@ let inline_mcodes =
     do_nothing do_nothing
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
     do_nothing do_nothing do_nothing_end do_nothing_end do_nothing do_nothing
-    do_nothing do_nothing do_nothing do_nothing do_nothing
+    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
 
 (* --------------------------------------------------------------------- *)
 (* For function declarations.  Can't use the mcode at the root, because that
@@ -274,7 +274,7 @@ let check_allminus =
     donothing
     donothing ident expression donothing donothing typeC initialiser donothing
     declaration field donothing statement donothing case_line donothing
-    donothing donothing
+    donothing donothing donothing
 
 (* --------------------------------------------------------------------- *)
 (* --------------------------------------------------------------------- *)
@@ -554,7 +554,7 @@ and typeC allminus t =
 	  List.map
 	    (function ty ->
 	      Ast.Type
-		(allminus, Some (mcode cv),
+		(allminus, List.map mcode cv,
 		 rewrap_iso ty (base_typeC allminus ty)))
 	    (collect_disjs ty) in
 	(* one could worry that isos are lost because we flatten the
@@ -570,7 +570,7 @@ and typeC allminus t =
     | Ast0.StructUnionDef(_,_,_,_) | Ast0.EnumDef(_,_,_,_)
     | Ast0.TypeOfExpr(_,_,_,_) | Ast0.TypeOfType(_,_,_,_)
     | Ast0.TypeName(_) | Ast0.AutoType(_) | Ast0.MetaType(_,_,_) ->
-	Ast.Type(allminus,None,rewrap t no_isos (base_typeC allminus t))
+        Ast.Type(allminus,[],rewrap t no_isos (base_typeC allminus t))
     | Ast0.DisjType(_,types,_,_) ->
 	Ast.DisjType(List.map (typeC allminus) types)
     | Ast0.ConjType(_,types,_,_) ->
@@ -632,23 +632,26 @@ and declaration d =
 	Ast.MetaDecl(mcode name,constraints cstr,unitary,false)
     | Ast0.AsDecl(decl,asdecl) ->
 	Ast.AsDecl(declaration decl,declaration asdecl)
-    | Ast0.Init(stg,ty,id,attr,eq,ini,sem) ->
+    | Ast0.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
 	let allminus = check_allminus.VT0.combiner_rec_declaration d in
 	let stg = get_option mcode stg in
 	let ty = typeC allminus ty in
 	let id = ident id in
-	let attr = List.map attribute attr in
+	let midattr = List.map attribute midattr in
+	let endattr = List.map attribute endattr in
 	let eq = mcode eq in
 	let ini = initialiser ini in
 	let sem = mcode sem in
-	Ast.Init(stg,ty,id,attr,eq,ini,sem)
-    | Ast0.UnInit(stg,ty,id,attr,sem) ->
+	Ast.Init(stg,ty,midattr,id,endattr,eq,ini,sem)
+    | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
 	let allminus = check_allminus.VT0.combiner_rec_declaration d in
-	let attr = List.map attribute attr in
-	Ast.UnInit(get_option mcode stg,typeC allminus ty,ident id,attr,
+	let midattr = List.map attribute midattr in
+	let endattr = List.map attribute endattr in
+	Ast.UnInit(get_option mcode stg,typeC allminus ty,midattr,ident id,endattr,
 		   mcode sem)
-    | Ast0.FunProto(fi,name,lp,params,va,rp,sem) ->
+    | Ast0.FunProto(fi,attr,name,lp,params,va,rp,sem) ->
 	  let fi = List.map fninfo fi in
+          let attr = List.map attribute attr in
 	  let name = ident name in
 	  let lp = mcode lp in
 	  let params = parameter_list params in
@@ -657,7 +660,7 @@ and declaration d =
             | Some (comma,ellipsis) -> Some(mcode comma,mcode ellipsis) in
 	  let rp = mcode rp in
 	  let sem = mcode sem in
-	  Ast.FunProto(fi,name,lp,params,va,rp,sem)
+	  Ast.FunProto(fi,attr,name,lp,params,va,rp,sem)
     | Ast0.MacroDecl(stg,name,lp,args,rp,attr,sem) ->
 	(* this would seem to need allminus... *)
 	let stg = get_option mcode stg in
@@ -687,7 +690,7 @@ and declaration d =
 	let allminus = check_allminus.VT0.combiner_rec_declaration d in
 	let id = typeC allminus id in
 	(match Ast.unwrap id with
-	  Ast.Type(_,None,id) -> (* only MetaType or Id *)
+          Ast.Type(_,[],id) -> (* only MetaType or Id *)
 	    Ast.Typedef(mcode stg,typeC allminus ty,id,mcode sem)
 	| _ -> failwith "bad typedef")
     | Ast0.DisjDecl(_,decls,_,_) -> Ast.DisjDecl(List.map declaration decls)
@@ -862,9 +865,9 @@ and parameterTypeDef p =
     (match Ast0.unwrap p with
       Ast0.VoidParam(ty,attr) ->
         Ast.VoidParam(typeC false ty,List.map attribute attr)
-    | Ast0.Param(ty,id,attr) ->
+    | Ast0.Param(ty,midattr,id,attr) ->
 	let allminus = check_allminus.VT0.combiner_rec_parameter p in
-	Ast.Param(typeC allminus ty,get_option ident id,List.map attribute attr)
+	Ast.Param(typeC allminus ty,List.map attribute midattr,get_option ident id,List.map attribute attr)
     | Ast0.MetaParam(name,cstr,_) ->
 	Ast.MetaParam(mcode name,constraints cstr,unitary,false)
     | Ast0.MetaParamList(name,lenname,cstr,_) ->
@@ -1036,7 +1039,7 @@ and statement s =
 		 (statement Ast.NotSequencible))
 	      whn in
 	  Ast.Dots(d,whn,[],[])
-      | Ast0.FunDecl((_,bef),fi,name,lp,params,va,rp,lbrace,body,rbrace,
+      | Ast0.FunDecl((_,bef),fi,name,lp,params,va,rp,attrs,lbrace,body,rbrace,
 		     (_,aft)) ->
 	  let fi = List.map fninfo fi in
 	  let name = ident name in
@@ -1046,6 +1049,7 @@ and statement s =
             | None -> None
             | Some (comma, ellipsis) -> Some (mcode comma, mcode ellipsis) in
 	  let rp = mcode rp in
+          let attrs = List.map attribute attrs in
           let lbrace = mcode lbrace in
           let body = dots (statement seqible) body in
           let rbrace = mcode rbrace in
@@ -1053,7 +1057,7 @@ and statement s =
           Ast.FunDecl(rewrap_rule_elem s
                         (Ast.FunHeader
                            (convert_allminus_mcodekind allminus bef,
-                            allminus,fi,name,lp,params,newva,rp)),
+                            allminus,fi,name,lp,params,newva,rp,attrs)),
                       tokenwrap lbrace s (Ast.SeqStart(lbrace)),
                       body,
                       tokenwrap rbrace s (Ast.SeqEnd(rbrace)),
@@ -1200,9 +1204,22 @@ and fninfo = function
 and attribute a =
   rewrap a no_isos
     (match Ast0.unwrap a with
-      Ast0.Attribute(attr) -> Ast.Attribute(mcode attr)
-    | Ast0.MetaAttribute(name,cstr,_) ->
-	Ast.MetaAttribute(mcode name,constraints cstr,unitary,false))
+      Ast0.Attribute(attr) -> Ast.Attribute(attr_arg attr)
+    | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+	Ast.GccAttribute(mcode attr_,mcode lp1,mcode lp2,attr_arg arg,mcode rp1,mcode rp2))
+
+and attr_arg a =
+  rewrap a no_isos
+    (match Ast0.unwrap a with
+      Ast0.MetaAttr(name,cstr,_) ->
+	Ast.MetaAttr(mcode name,constraints cstr,unitary,false)
+    | Ast0.MacroAttr(arg) -> Ast.MacroAttr(mcode arg)
+    | Ast0.MacroAttrArgs(attr,lp,args,rp) ->
+        let attr = mcode attr in
+	let lp = mcode lp in
+	let args = dots expression args in
+	let rp = mcode rp in
+        Ast.MacroAttrArgs(attr,lp,args,rp))
 
 and option_to_list = function
     Some x -> [x]
@@ -1267,6 +1284,7 @@ and anything = function
   | Ast0.CaseLineTag(d) -> Ast.CaseLineTag(case_line d)
   | Ast0.StringFragmentTag(d) -> Ast.StringFragmentTag(string_fragment d)
   | Ast0.AttributeTag(d) -> Ast.AttributeTag(attribute d)
+  | Ast0.AttrArgTag(d) -> Ast.AttrArgTag(attr_arg d)
   | Ast0.TopTag(d) -> Ast.Code(top_level d)
   | Ast0.IsoWhenTag(_) -> failwith "not possible"
   | Ast0.IsoWhenTTag(_) -> failwith "not possible"
