@@ -313,7 +313,7 @@ and isWchar = IsWchar | IsUchar | Isuchar | Isu8char | IsChar
 
 and base_fullType =
     Type            of bool (* true if all minus *) *
-                       const_vol mcode option * typeC
+                       const_vol mcode list * typeC
   | AsType          of fullType * fullType (* as type, always metavar *)
   | DisjType        of fullType list
   | ConjType        of fullType list
@@ -370,12 +370,12 @@ and const_vol = Const | Volatile
    split out into multiple declarations of a single variable each. *)
 
 and base_declaration =
-    Init of storage mcode option * fullType * ident * attr list *
-	string mcode (*=*) * initialiser * string mcode (*;*)
-  | UnInit of storage mcode option * fullType * ident * attr list *
-	string mcode (* ; *)
+  | Init of storage mcode option * fullType * attr list * ident *
+        attr list * string mcode (*=*) * initialiser * string mcode (*;*)
+  | UnInit of storage mcode option * fullType * attr list * ident *
+	attr list * string mcode (* ; *)
   | FunProto of
-	fninfo list * ident (* name *) *
+	fninfo list * attr list * ident (* name *) *
 	string mcode (* ( *) * parameter_list *
 	(string mcode (* , *) * string mcode (* ...... *) ) option *
 	string mcode (* ) *) * string mcode (* ; *)
@@ -471,7 +471,7 @@ and initialiser = base_initialiser wrap
 
 and base_parameterTypeDef =
     VoidParam     of fullType * attr list
-  | Param         of fullType * ident option * attr list
+  | Param         of fullType * attr list * ident option * attr list
 
   | MetaParam     of meta_name mcode * constraints * keep_binding * inherited
   | MetaParamList of meta_name mcode * listlen * constraints * keep_binding *
@@ -533,7 +533,7 @@ and base_rule_elem =
 	             fninfo list * ident (* name *) *
 	             string mcode (* ( *) * parameter_list *
                      (string mcode (* , *) * string mcode (* ...... *) ) option *
-                     string mcode (* ) *)
+                     string mcode (* ) *) * attr list
   | Decl          of annotated_decl
 
   | SeqStart      of string mcode (* { *)
@@ -606,11 +606,21 @@ and fninfo =
   | FAttr of attr
 
 and base_attr =
-    Attribute of string mcode
-  | MetaAttribute of meta_name mcode * constraints * keep_binding * inherited
+    Attribute of attr_arg
+  | GccAttribute of string mcode (* __attribute__ *) *
+                    string mcode (* ( *) * string mcode (* ( *) *
+                    attr_arg * string mcode (* ) *) * string mcode (* ) *)
 
 and attr = base_attr wrap
 
+and base_attr_arg =
+    MacroAttr of string mcode
+  | MacroAttrArgs of string mcode * string mcode (* ( *) *
+                      expression dots * string mcode (* ) *)
+  | MetaAttr of meta_name mcode * constraints * keep_binding * inherited
+
+and attr_arg = base_attr_arg wrap
+
 and metaStmtInfo =
     NotSequencible | SequencibleAfterDots of dots_whencode list | Sequencible
 
@@ -776,6 +786,7 @@ and anything =
   | CaseLineTag         of case_line
   | StringFragmentTag   of string_fragment
   | AttributeTag        of attr
+  | AttrArgTag          of attr_arg
   | ConstVolTag         of const_vol
   | Token               of string * info option
   | Directive           of added_string list
@@ -867,7 +878,7 @@ val string_of_binaryOp : binaryOp -> string
 
 val string_of_sign : sign -> string
 val string_of_baseType : baseType -> string
-val string_of_const_vol : const_vol -> string
+val string_of_const_vol : const_vol list -> string
 val string_of_structUnion : structUnion -> string
 val string_of_typeC : typeC -> string
 val string_of_fullType : fullType -> string
