@@ -545,9 +545,9 @@ and string_format e =
 and typeC t =
   match Ast0.unwrap t with
     Ast0.ConstVol(cv,ty) ->
-      let cv = normal_mcode cv in
+      let cv = List.map normal_mcode cv in
       let ty = typeC ty in
-      mkres t (Ast0.ConstVol(cv,ty)) (promote_mcode cv) ty
+      mkres t (Ast0.ConstVol(cv,ty)) (promote_mcode (List.hd cv)) ty
   | Ast0.BaseType(ty,strings) ->
       let strings = List.map normal_mcode strings in
       let first = List.hd strings in
@@ -669,37 +669,40 @@ and declaration d =
       let name = normal_mcode name in
       let ln = promote_mcode name in
       mkres d (Ast0.MetaDecl(name,a,b)) ln ln
-  | Ast0.Init(stg,ty,id,attr,eq,exp,sem) ->
+  | Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem) ->
       let ty = typeC ty in
+      let midattr = List.map attribute midattr in
       let id = ident id in
-      let attr = List.map attribute attr in
+      let endattr = List.map attribute endattr in
       let eq = normal_mcode eq in
       let exp = initialiser exp in
       let sem = normal_mcode sem in
       (match stg with
 	None ->
-	  mkres d (Ast0.Init(stg,ty,id,attr,eq,exp,sem)) ty (promote_mcode sem)
+	  mkres d (Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem)) ty (promote_mcode sem)
       | Some x ->
 	  let stg = Some (normal_mcode x) in
-	  mkres d (Ast0.Init(stg,ty,id,attr,eq,exp,sem))
+	  mkres d (Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem))
 	    (promote_mcode x) (promote_mcode sem))
-  | Ast0.UnInit(stg,ty,id,attr,sem) ->
+  | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
       let ty = typeC ty in
+      let midattr = List.map attribute midattr in
       let id = ident id in
-      let attr = List.map attribute attr in
+      let endattr = List.map attribute endattr in
       let sem = normal_mcode sem in
       (match stg with
 	None ->
-	  mkres d (Ast0.UnInit(stg,ty,id,attr,sem)) ty (promote_mcode sem)
+	  mkres d (Ast0.UnInit(stg,ty,midattr,id,endattr,sem)) ty (promote_mcode sem)
       | Some x ->
 	  let stg = Some (normal_mcode x) in
-	  mkres d (Ast0.UnInit(stg,ty,id,attr,sem))
+	  mkres d (Ast0.UnInit(stg,ty,midattr,id,endattr,sem))
 	    (promote_mcode x) (promote_mcode sem))
-  | Ast0.FunProto(fninfo,name,lp1,params,va1,rp1,sem) ->
+  | Ast0.FunProto(fninfo,attr,name,lp1,params,va1,rp1,sem) ->
       let fninfo =
 	List.map
 	  (function Ast0.FType(ty) -> Ast0.FType(typeC ty) | x -> x)
 	  fninfo in
+      let attr = List.map attribute attr in
       let name = ident name in
       let lp1 = normal_mcode lp1 in
       let params = parameter_list (Some(promote_mcode lp1)) params in
@@ -708,7 +711,7 @@ and declaration d =
         | Some (c1,e1) -> Some (normal_mcode c1, normal_mcode e1) in
       let rp1 = normal_mcode rp1 in
       let sem = normal_mcode sem in
-      let res = Ast0.FunProto(fninfo,name,lp1,params,va1,rp1,sem) in
+      let res = Ast0.FunProto(fninfo,attr,name,lp1,params,va1,rp1,sem) in
       let right = promote_mcode sem in
       (match fninfo with
 	  [] -> mkres d res name right
@@ -910,13 +913,34 @@ and initialiser_dots x = dots is_init_dots None initialiser x
 and attribute attr =
   match Ast0.unwrap attr with
     Ast0.Attribute(a) ->
+      let a = attr_arg a in
+      mkres attr (Ast0.Attribute(a)) a a
+  | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+      let attr_ = normal_mcode attr_ in
+      let lp1 = normal_mcode lp1 in
+      let lp2 = normal_mcode lp2 in
+      let arg = attr_arg arg in
+      let rp1 = normal_mcode rp1 in
+      let rp2 = normal_mcode rp2 in
+      let ln1 = promote_mcode attr_ in
+      let ln2 = promote_mcode rp2 in
+      mkres attr (Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2)) ln1 ln2
+
+and attr_arg arg =
+  match Ast0.unwrap arg with
+    Ast0.MacroAttr(a) ->
       let ln = promote_mcode a in
-      mkres attr (Ast0.Attribute(a)) ln ln
-  | Ast0.MetaAttribute(name,a,b) ->
+      mkres arg (Ast0.MacroAttr(a)) ln ln
+  | Ast0.MetaAttr(name,a,b) ->
       let name = normal_mcode name in
       let ln = promote_mcode name in
-      mkres attr (Ast0.MetaAttribute(name,a,b)) ln ln
-
+      mkres arg (Ast0.MetaAttr(name,a,b)) ln ln
+  | Ast0.MacroAttrArgs(attr,lp,args,rp) ->
+      let attr = normal_mcode attr in
+      let lp = normal_mcode lp in
+      let args = dots is_exp_dots (Some(promote_mcode lp)) expression args in
+      let rp = normal_mcode rp in
+      mkres arg (Ast0.MacroAttrArgs(attr,lp,args,rp)) (promote_mcode attr) (promote_mcode rp)
 
 (* --------------------------------------------------------------------- *)
 (* Parameter *)
@@ -936,23 +960,25 @@ and parameterTypeDef p =
       | l ->
           let lattr = List.hd (List.rev l) in
           mkres p (Ast0.VoidParam(ty,attr)) ty lattr)
-  | Ast0.Param(ty,Some id,attr) ->
+  | Ast0.Param(ty,midattr,Some id,attr) ->
       let id = ident id in
+      let midattr = List.map attribute midattr in
       let ty = typeC ty in
       let attr = List.map attribute attr in
       (match attr with
-        [] -> mkres p (Ast0.Param(ty,Some id,attr)) ty id
+        [] -> mkres p (Ast0.Param(ty,midattr,Some id,attr)) ty id
       | l ->
           let lattr = List.hd (List.rev l) in
-          mkres p (Ast0.Param(ty,Some id,attr)) ty lattr)
-  | Ast0.Param(ty,None,attr) ->
+          mkres p (Ast0.Param(ty,midattr,Some id,attr)) ty lattr)
+  | Ast0.Param(ty,midattr,None,attr) ->
+      assert (midattr = []);
       let attr = List.map attribute attr in
       let ty = typeC ty in
       (match attr with
-        [] -> mkres p (Ast0.Param(ty,None,attr)) ty ty
+        [] -> mkres p (Ast0.Param(ty,[],None,attr)) ty ty
       | l ->
           let lattr = List.hd (List.rev l) in
-          mkres p (Ast0.Param(ty,None,attr)) ty lattr)
+          mkres p (Ast0.Param(ty,[],None,attr)) ty lattr)
   | Ast0.MetaParam(name,a,b) ->
       let name = normal_mcode name in
       let ln = promote_mcode name in
@@ -1275,7 +1301,7 @@ let rec statement s =
 	let dots = bad_mcode dots in
 	let ln = promote_mcode dots in
 	mkres s (Ast0.Dots(dots,whencode)) ln ln
-    | Ast0.FunDecl((_,bef),fninfo,name,lp,params,va,rp,lbrace,body,rbrace,
+    | Ast0.FunDecl((_,bef),fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,
 		   (_,aft)) ->
 	let fninfo =
 	  List.map
@@ -1285,6 +1311,7 @@ let rec statement s =
 	let lp = normal_mcode lp in
 	let params = parameter_list (Some(promote_mcode lp)) params in
 	let rp = normal_mcode rp in
+	let attrs = List.map attribute attrs in
 	let lbrace = normal_mcode lbrace in
 	let body =
 	  dots is_stm_dots (Some(promote_mcode lbrace)) statement body in
@@ -1299,7 +1326,7 @@ let rec statement s =
 	 and other things to the node after, but that would complicate
 	 insert_plus, which doesn't distinguish between different mcodekinds *)
 	let res =
-	  Ast0.FunDecl((leftinfo,bef),fninfo,name,lp,params,va,rp,lbrace,
+	  Ast0.FunDecl((leftinfo,bef),fninfo,name,lp,params,va,rp,attrs,lbrace,
 		       body,rbrace,(rightinfo,aft)) in
       (* have to do this test again, because of typing problems - can't save
 	 the result, only use it *)
