@@ -47,6 +47,7 @@ let set_mcodekind x mcodekind =
   | Ast0.CaseLineTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.StringFragmentTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.AttributeTag(d) -> Ast0.set_mcodekind d mcodekind
+  | Ast0.AttrArgTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.TopTag(d) -> Ast0.set_mcodekind d mcodekind
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
@@ -83,6 +84,7 @@ let set_index x index =
   | Ast0.CaseLineTag(d) -> Ast0.set_index d index
   | Ast0.StringFragmentTag(d) -> Ast0.set_index d index
   | Ast0.AttributeTag(d) -> Ast0.set_index d index
+  | Ast0.AttrArgTag(d) -> Ast0.set_index d index
   | Ast0.TopTag(d) -> Ast0.set_index d index
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
@@ -118,6 +120,7 @@ let get_index = function
   | Ast0.CaseLineTag(d) -> Index.case_line d
   | Ast0.StringFragmentTag(d) -> Index.string_fragment d
   | Ast0.AttributeTag(d) -> Index.attribute d
+  | Ast0.AttrArgTag(d) -> Index.attr_arg d
   | Ast0.TopTag(d) -> Index.top_level d
   | Ast0.IsoWhenTag(_) -> failwith "only within iso phase"
   | Ast0.IsoWhenTTag(_) -> failwith "only within iso phase"
@@ -169,7 +172,7 @@ let collect_plus_lines top =
 	(* cases for everything with extra mcode *)
       | Ast0.Decl((info,bef),_) ->
 	  bind (mcode info bef) (k s)
-      |	Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
+      |	Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
 	  bind (mcode info bef) (bind (k s) (mcode ainfo aft))
       | Ast0.IfThen(_,_,_,_,_,(info,aft,adj))
       | Ast0.IfThenElse(_,_,_,_,_,_,_,(info,aft,adj))
@@ -186,7 +189,7 @@ let collect_plus_lines top =
       donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing statement donothing donothing donothing
-      donothing donothing in
+      donothing donothing donothing in
   fn.VT0.combiner_rec_top_level top
 
 (* --------------------------------------------------------------------- *)
@@ -422,26 +425,34 @@ let classify is_minus all_marked table code =
 	   term structure.  In (all?) other such cases, we visit the terms
 	   using rebuilder, which just visits the subterms, rather than
 	   reordering their components. *)
-      |	Ast0.Init(stg,ty,id,attr,eq,ini,sem) ->
+      |	Ast0.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
 	  bind (match stg with Some stg -> mcode stg | _ -> option_default)
 	    (bind (r.VT0.combiner_rec_typeC ty)
-	       (bind (r.VT0.combiner_rec_ident id)
-                  (bind
-                     (List.fold_right bind
-                       (List.map r.VT0.combiner_rec_attribute attr)
-			option_default)
-		     (bind (mcode eq)
-			(bind (r.VT0.combiner_rec_initialiser ini)
-			   (mcode sem))))))
-      | Ast0.UnInit(stg,ty,id,attr,sem) ->
+              (bind
+                 (List.fold_right bind
+                   (List.map r.VT0.combiner_rec_attribute midattr)
+                    option_default)
+                 (bind (r.VT0.combiner_rec_ident id)
+                    (bind
+                       (List.fold_right bind
+                         (List.map r.VT0.combiner_rec_attribute endattr)
+                          option_default)
+                       (bind (mcode eq)
+                          (bind (r.VT0.combiner_rec_initialiser ini)
+                             (mcode sem)))))))
+      | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
 	  bind (match stg with Some stg -> mcode stg | _ -> option_default)
 	    (bind (r.VT0.combiner_rec_typeC ty)
-	       (bind (r.VT0.combiner_rec_ident id)
-                  (bind
-                     (List.fold_right bind
-                       (List.map r.VT0.combiner_rec_attribute attr)
-			option_default)
-		     (mcode sem))))
+              (bind
+                 (List.fold_right bind
+                   (List.map r.VT0.combiner_rec_attribute midattr)
+                    option_default)
+                 (bind (r.VT0.combiner_rec_ident id)
+                    (bind
+                       (List.fold_right bind
+                         (List.map r.VT0.combiner_rec_attribute endattr)
+                          option_default)
+                       (mcode sem)))))
       |	_ -> k e) in
 
   let field r k e =
@@ -472,13 +483,16 @@ let classify is_minus all_marked table code =
   let param r k e =
     compute_result Ast0.param e
       (match Ast0.unwrap e with
-	Ast0.Param(ty,Some id,attr) ->
+	Ast0.Param(ty,midattr,Some id,attr) ->
 	  (* needed for the same reason as in the Init and UnInit cases *)
 	  bind (r.VT0.combiner_rec_typeC ty)
-           (bind (r.VT0.combiner_rec_ident id)
-              (List.fold_right bind
-                 (List.map r.VT0.combiner_rec_attribute attr)
-                 option_default))
+            (bind (List.fold_right bind
+              (List.map r.VT0.combiner_rec_attribute midattr)
+              option_default)
+              (bind (r.VT0.combiner_rec_ident id)
+                (List.fold_right bind
+                   (List.map r.VT0.combiner_rec_attribute attr)
+                   option_default)))
       |	_ -> k e) in
 
   let typeC r k e =
@@ -518,7 +532,7 @@ let classify is_minus all_marked table code =
 	(* cases for everything with extra mcode *)
       | Ast0.Decl((info,bef),_) ->
 	  bind (nc_mcode ((),(),info,bef,(),-1)) (k s)
-      | Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
+      | Ast0.FunDecl((info,bef),_,_,_,_,_,_,_,_,_,_,(ainfo,aft)) ->
 	  (* not sure that the use of start is relevant here *)
 	  let a1 = nc_mcode ((),(),info,bef,(),-1) in
 	  let a2 = nc_mcode ((),(),ainfo,aft,(),-1) in
@@ -558,7 +572,7 @@ let classify is_minus all_marked table code =
       ident expression (do_nothing Ast0.assignOp) (do_nothing Ast0.binaryOp)
       typeC initialiser param declaration field enum_decl
       statement (do_nothing Ast0.forinfo) case_line string_fragment
-      (do_nothing Ast0.attr) (do_top Ast0.top) in
+      (do_nothing Ast0.attr) (do_nothing Ast0.attr_arg) (do_top Ast0.top) in
   combiner.VT0.combiner_rec_top_level code
 
 (* --------------------------------------------------------------------- *)
@@ -593,12 +607,25 @@ let equal_option e1 e2 =
 let dots fn d1 d2 =
   List.length (Ast0.unwrap d1) = List.length (Ast0.unwrap d2)
 
+let equal_attr_arg a1 a2 =
+  match (Ast0.unwrap a1, Ast0.unwrap a2) with
+    (Ast0.MacroAttr(name1),Ast0.MacroAttr(name2)) ->
+      equal_mcode name1 name2
+  | (Ast0.MacroAttrArgs(attr1,lp1,args1,rp1),Ast0.MacroAttrArgs(attr2,lp2,args2,rp2)) ->
+      equal_mcode attr1 attr2 && equal_mcode lp1 lp2 && equal_mcode rp1 rp2
+  | (Ast0.MetaAttr(name1,_,_),Ast0.MetaAttr(name2,_,_)) ->
+      equal_mcode name1 name2
+  | _ -> false
+
 let equal_attribute a1 a2 =
   match (Ast0.unwrap a1, Ast0.unwrap a2) with
     (Ast0.Attribute(attr1),Ast0.Attribute(attr2)) ->
-      equal_mcode attr1 attr2
-  | (Ast0.MetaAttribute(name1,_,_),Ast0.MetaAttribute(name2,_,_)) ->
-      equal_mcode name1 name2
+      equal_attr_arg attr1 attr2
+  | (Ast0.GccAttribute(attr_1,lp11,lp21,arg1,rp11,rp21),
+     Ast0.GccAttribute(attr_2,lp12,lp22,arg2,rp12,rp22)) ->
+      equal_mcode attr_1 attr_2 && equal_mcode lp11 lp12 &&
+      equal_mcode lp21 lp22 && equal_attr_arg arg1 arg2 &&
+      equal_mcode rp11 rp12 && equal_mcode rp21 rp22
   | _ -> false
 
 let equal_ident i1 i2 =
@@ -694,7 +721,7 @@ and binaryOp_equal op1 op2 =
 
 let equal_typeC t1 t2 =
   match (Ast0.unwrap t1,Ast0.unwrap t2) with
-    (Ast0.ConstVol(cv1,_),Ast0.ConstVol(cv2,_)) -> equal_mcode cv1 cv2
+    (Ast0.ConstVol(cv1,_),Ast0.ConstVol(cv2,_)) -> List.for_all2 equal_mcode cv1 cv2
   | (Ast0.BaseType(ty1,stringsa),Ast0.BaseType(ty2,stringsb)) ->
       List.for_all2 equal_mcode stringsa stringsb
   | (Ast0.Signed(sign1,_),Ast0.Signed(sign2,_)) ->
@@ -755,19 +782,23 @@ let equal_declaration d1 d2 =
   match (Ast0.unwrap d1,Ast0.unwrap d2) with
     (Ast0.MetaDecl(name1,_,_),Ast0.MetaDecl(name2,_,_)) ->
       equal_mcode name1 name2
-  | (Ast0.Init(stg1,_,_,attr1,eq1,_,sem1),
-     Ast0.Init(stg2,_,_,attr2,eq2,_,sem2)) ->
+  | (Ast0.Init(stg1,_,midattr1,_,endattr1,eq1,_,sem1),
+     Ast0.Init(stg2,_,midattr2,_,endattr2,eq2,_,sem2)) ->
       equal_option stg1 stg2 &&
-      (List.length attr1) = (List.length attr2) &&
-      List.for_all2 equal_attribute attr1 attr2 &&
+      (List.length midattr1) = (List.length midattr2) &&
+      List.for_all2 equal_attribute midattr1 midattr2 &&
+      (List.length endattr1) = (List.length endattr2) &&
+      List.for_all2 equal_attribute endattr1 endattr2 &&
       equal_mcode eq1 eq2 && equal_mcode sem1 sem2
-  | (Ast0.UnInit(stg1,_,_,attr1,sem1),Ast0.UnInit(stg2,_,_,attr2,sem2)) ->
+  | (Ast0.UnInit(stg1,_,midattr1,_,endattr1,sem1),Ast0.UnInit(stg2,_,midattr2,_,endattr2,sem2)) ->
       equal_option stg1 stg2 &&
-      (List.length attr1) = (List.length attr2) &&
-      List.for_all2 equal_attribute attr1 attr2 &&
+      (List.length midattr1) = (List.length midattr2) &&
+      List.for_all2 equal_attribute midattr1 midattr2 &&
+      (List.length endattr1) = (List.length endattr2) &&
+      List.for_all2 equal_attribute endattr1 endattr2 &&
       equal_mcode sem1 sem2
-  | (Ast0.FunProto(fninfo1,name1,lp1,p1,va1,rp1,sem1),
-     Ast0.FunProto(fninfo2,name2,lp2,p2,va2,rp2,sem2)) ->
+  | (Ast0.FunProto(fninfo1,attr1,name1,lp1,p1,va1,rp1,sem1),
+     Ast0.FunProto(fninfo2,attr2,name2,lp2,p2,va2,rp2,sem2)) ->
        let equal_varargs va1 va2 = match (va1,va2) with
          | None, None -> true
          | Some (c1, e1), Some (c2, e2) ->
@@ -775,6 +806,8 @@ let equal_declaration d1 d2 =
          | _ -> false in
        (List.length fninfo1) = (List.length fninfo2) &&
        List.for_all2 equal_fninfo fninfo1 fninfo2 &&
+      (List.length attr1) = (List.length attr2) &&
+      List.for_all2 equal_attribute attr1 attr2 &&
        equal_mcode lp1 lp2 && equal_varargs va1 va2 &&
        equal_mcode rp1 rp2 && equal_mcode sem1 sem2
   | (Ast0.MacroDecl(stg1,nm1,lp1,_,rp1,attr1,sem1),
@@ -871,9 +904,11 @@ let equal_parameterTypeDef p1 p2 =
     (Ast0.VoidParam(_,ar1),Ast0.VoidParam(_,ar2)) ->
       (List.length ar1) = (List.length ar2) &&
       List.for_all2 equal_attribute ar1 ar2
-  | (Ast0.Param(_,_,ar1),Ast0.Param(_,_,ar2)) ->
+  | (Ast0.Param(_,mar1,_,ar1),Ast0.Param(_,mar2,_,ar2)) ->
       (List.length ar1) = (List.length ar2) &&
-      List.for_all2 equal_attribute ar1 ar2
+      List.for_all2 equal_attribute ar1 ar2 &&
+      (List.length mar1) = (List.length mar2) &&
+      List.for_all2 equal_attribute mar1 mar2
   | (Ast0.MetaParam(name1,_,_),Ast0.MetaParam(name2,_,_))
   | (Ast0.MetaParamList(name1,_,_,_),Ast0.MetaParamList(name2,_,_,_)) ->
       equal_mcode name1 name2
@@ -884,8 +919,8 @@ let equal_parameterTypeDef p1 p2 =
 
 let equal_statement s1 s2 =
   match (Ast0.unwrap s1,Ast0.unwrap s2) with
-    (Ast0.FunDecl(_,fninfo1,_,lp1,_,_,rp1,lbrace1,_,rbrace1,_),
-     Ast0.FunDecl(_,fninfo2,_,lp2,_,_,rp2,lbrace2,_,rbrace2,_)) ->
+    (Ast0.FunDecl(_,fninfo1,_,lp1,_,_,rp1,_,lbrace1,_,rbrace1,_),
+     Ast0.FunDecl(_,fninfo2,_,lp2,_,_,rp2,_,lbrace2,_,rbrace2,_)) ->
        (List.length fninfo1) = (List.length fninfo2) &&
        List.for_all2 equal_fninfo fninfo1 fninfo2 &&
        equal_mcode lp1 lp2 && equal_mcode rp1 rp2 &&
@@ -1069,7 +1104,7 @@ let contextify_all =
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing
+    donothing donothing donothing donothing donothing
 
 let contextify_whencode =
   let bind x y = () in
@@ -1216,14 +1251,14 @@ let rec is_init s =
 let rec is_decl s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
-  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
+  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_decl stmts
   | Ast0.Conj(_,stmts,_,_) -> isany is_decl stmts
   | _ -> false
 
 let rec is_fndecl s =
   match Ast0.unwrap s with
-    Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
+    Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_fndecl stmts
   | Ast0.Conj(_,stmts,_,_) -> isany is_fndecl stmts
   | _ -> false
@@ -1231,7 +1266,7 @@ let rec is_fndecl s =
 let rec is_toplevel s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
-  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
+  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_toplevel stmts
   | Ast0.Conj(_,stmts,_,_) -> isany is_toplevel stmts
   | Ast0.ExprStatement(Some fc,_) ->
