@@ -261,9 +261,9 @@ and disjexp e =
 and disjparam p =
   match Ast.unwrap p with
     Ast.VoidParam(ty,attr) -> [p] (* void is the only possible value *)
-  | Ast.Param(ty,id,attr) ->
+  | Ast.Param(ty,midattr,id,attr) ->
       disjmult2 (disjty ty) (disjoption disjident id)
-	(fun ty id -> Ast.rewrap p (Ast.Param(ty,id,attr)))
+	(fun ty id -> Ast.rewrap p (Ast.Param(ty,midattr,id,attr)))
   | Ast.AsParam(pm,asexp) -> (* as exp doesn't contain disj *)
       let pm = disjparam pm in
       List.map (function pm -> Ast.rewrap p (Ast.AsParam(pm,asexp))) pm
@@ -331,18 +331,18 @@ and disjdecl d =
   | Ast.AsDecl(decl,asdecl) ->
       let decl = disjdecl decl in
       List.map (function decl -> Ast.rewrap d (Ast.AsDecl(decl,asdecl))) decl
-  | Ast.Init(stg,ty,id,attr,eq,ini,sem) ->
+  | Ast.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
       disjmult3 (disjty ty) (disjident id) (disjini ini)
 	(fun ty id ini ->
-	  Ast.rewrap d (Ast.Init(stg,ty,id,attr,eq,ini,sem)))
-  | Ast.UnInit(stg,ty,id,attr,sem) ->
+	  Ast.rewrap d (Ast.Init(stg,ty,midattr,id,endattr,eq,ini,sem)))
+  | Ast.UnInit(stg,ty,midattr,id,endattr,sem) ->
       disjmult2 (disjty ty) (disjident id)
 	(fun ty id ->
-	  Ast.rewrap d (Ast.UnInit(stg,ty,id,attr,sem)))
-  | Ast.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
+	  Ast.rewrap d (Ast.UnInit(stg,ty,midattr,id,endattr,sem)))
+  | Ast.FunProto(fninfo,attr,name,lp1,params,va,rp1,sem) ->
       disjmult2 (disjmult disjfninfo fninfo) (disjident name)
 	(fun fninfo name ->
-	  Ast.rewrap d (Ast.FunProto(fninfo,name,lp1,params,va,rp1,sem)))
+	  Ast.rewrap d (Ast.FunProto(fninfo,attr,name,lp1,params,va,rp1,sem)))
   | Ast.MacroDecl(stg,name,lp,args,rp,attr,sem) ->
       disjmult2 (disjident name) (disjdots disjexp args)
 	(fun name args ->
@@ -393,13 +393,13 @@ let orify_rule_elem_ini = generic_orify_rule_elem disjini
 
 let rec disj_rule_elem r k re =
   match Ast.unwrap re with
-    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp) ->
+    Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp,attrs) ->
       generic_orify_rule_elem
 	(disjtwoelems disjident (disjdots disjparam)) re
 	(name,params)
 	(fun (name,params) ->
 	  Ast.rewrap re
-	    (Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp)))
+	    (Ast.FunHeader(bef,allminus,fninfo,name,lp,params,va,rp,attrs)))
   | Ast.Decl decl ->
       orify_rule_elem_anndecl re decl
 	(function decl -> Ast.rewrap re (Ast.Decl decl))
@@ -483,7 +483,7 @@ let disj_all =
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing disj_rule_elem donothing donothing donothing donothing
-    donothing
+    donothing donothing
 
 (* ----------------------------------------------------------------------- *)
 (* collect iso information at the rule_elem level *)
@@ -501,7 +501,7 @@ let collect_all_isos =
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing
     doanything donothing doanything donothing donothing donothing donothing
-    donothing donothing doanything
+    donothing donothing donothing doanything
 
 let collect_iso_info =
   let mcode x = x in
@@ -519,7 +519,7 @@ let collect_iso_info =
     donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing rule_elem donothing donothing
-    donothing donothing donothing
+    donothing donothing donothing donothing
 
 (* ----------------------------------------------------------------------- *)
 
