@@ -38,7 +38,7 @@ let make_semi info =
 let collect_function (stm : Ast0.statement) =
   match Ast0.unwrap stm with
     Ast0.FunDecl((bef_info,_),
-		 fninfo,name,lp,params,va,rp,lbrace,body,rbrace,
+		 fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,
 		 (aft_info,_)) ->
       let new_bef_info =
 	{(Ast0.default_info()) with
@@ -50,7 +50,7 @@ let collect_function (stm : Ast0.statement) =
 	     (Ast0.Decl((new_bef_info,Ast0.context_befaft()),
 			Ast0.copywrap stm
 			  (Ast0.FunProto
-			     (fninfo,name,lp,params,None,rp,make_semi aft_info))))))
+                             (fninfo,[],name,lp,params,None,rp,make_semi aft_info))))))
 	(get_name name)
   | _ -> []
 
@@ -68,7 +68,7 @@ let drop_positions =
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
     donothing donothing donothing donothing donothing donothing donothing
-    donothing donothing donothing donothing in
+    donothing donothing donothing donothing donothing in
   res.VT0.rebuilder_rec_statement
 
 let get_all_functions rule =
@@ -167,12 +167,12 @@ and strip =
 	     Ast0.MetaParamList(nm,lenname,cstr,Ast0.Pure)
 	 | e -> e)) in
 
-  let attribute r k e =
+  let attr_arg r k e =
     donothing r k
       (Ast0.rewrap e
 	 (match Ast0.unwrap e with
-	   Ast0.MetaAttribute(nm,cstr,pure) ->
-	     Ast0.MetaAttribute(nm,cstr,Ast0.Pure)
+	   Ast0.MetaAttr(nm,cstr,pure) ->
+	     Ast0.MetaAttr(nm,cstr,Ast0.Pure)
 	 | e -> e)) in
 
   V0.flat_rebuilder
@@ -182,7 +182,7 @@ and strip =
     donothing donothing
     ident donothing donothing donothing typeC donothing param
     donothing donothing donothing donothing donothing donothing donothing
-    attribute donothing
+    donothing attr_arg donothing
 
 and changed_proto = function
     (mname,mdef,mproto,None) -> true
@@ -206,7 +206,7 @@ let collect_ident_strings id =
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
-      donothing donothing donothing donothing in
+      donothing donothing donothing donothing donothing in
       v.VT0.combiner_rec_ident id
 
 let right_attach_mcode strings (x,ar,info,mc,pos,adj) =
@@ -269,10 +269,10 @@ let rec attach_right strings ty =
 let rec drop_param_name p =
   Ast0.rewrap p
     (match Ast0.unwrap p with
-      Ast0.Param(p,Some id,attr) ->
+      Ast0.Param(p,midattr,Some id,attr) ->
 	let strings = collect_ident_strings id in
 	let p = attach_right strings p in
-	Ast0.Param(p,None,attr)
+        Ast0.Param(p,[],None,midattr@attr)
     | Ast0.OptParam(p) -> Ast0.OptParam(drop_param_name p)
     | p -> p)
 
@@ -281,7 +281,7 @@ let drop_names dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
 	  let params =
 	    Ast0.rewrap params
 	      (List.map drop_param_name (Ast0.unwrap params)) in
@@ -289,7 +289,7 @@ let drop_names dec =
 	    (Ast0.Decl
 	       (info,
 		Ast0.rewrap proto
-		  (Ast0.FunProto(fninfo,name,lp,params,va,rp,sem))))
+		  (Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem))))
       |	_ -> failwith "unexpected declaration")
   | _ -> failwith "unexpected term"
 
@@ -305,7 +305,7 @@ let new_iname name index =
 
 let rec rename_param old_name all param index =
   match Ast0.unwrap param with
-    Ast0.Param(ty,Some id,attr) when all ->
+    Ast0.Param(ty,midattr,Some id,attr) when all ->
       (match Ast0.unwrap id with
 	Ast0.MetaId
 	  (((_,name),arity,info,mcodekind,pos,adj),constraints,seed,pure) ->
@@ -316,7 +316,7 @@ let rec rename_param old_name all param index =
 		 ((nm,arity,info,mcodekind,pos,adj),constraints,seed,
 		  Ast0.Pure)) in
 	  ([Ast.MetaIdDecl(Ast.NONE,nm)],
-	   Ast0.rewrap param (Ast0.Param(ty,Some new_id,attr)))
+	   Ast0.rewrap param (Ast0.Param(ty,midattr,Some new_id,attr)))
       |	_ -> ([],param))
   | Ast0.Pdots(d) ->
       let nm = (old_name,new_iname "__P" index) in
@@ -354,7 +354,7 @@ let fresh_names old_name mdef dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
 	  let (metavars,newdec) =
 	    let (metavars,l) =
 	      let params = Ast0.unwrap params in
@@ -367,10 +367,10 @@ let fresh_names old_name mdef dec =
 		  (info,
 		   Ast0.rewrap proto
 		     (Ast0.FunProto
-			(fninfo,name,lp,Ast0.rewrap params l,va,rp,sem))))) in
+			(fninfo,attr,name,lp,Ast0.rewrap params l,va,rp,sem))))) in
 	  let (def_metavars,newdef) =
 	    match Ast0.unwrap mdef with
-	      Ast0.FunDecl(x,fninfo,name,lp,params,va,rp,lb,body,rb,y) ->
+	      Ast0.FunDecl(x,fninfo,name,lp,params,va,rp,attrs,lb,body,rb,y) ->
 		let (def_metavars,def_l) =
 		  let params = Ast0.unwrap params in
 		  List.split
@@ -379,7 +379,7 @@ let fresh_names old_name mdef dec =
 		(List.concat def_metavars,
 		 Ast0.rewrap mdef
 		   (Ast0.FunDecl(x,fninfo,name,lp,Ast0.rewrap params def_l,va,
-				 rp,lb,body,rb,y)))
+				 rp,attrs,lb,body,rb,y)))
 	    | _ -> failwith "unexpected function definition" in
 	  (metavars,def_metavars,newdec,newdef)
       |	_ -> res)
@@ -390,7 +390,7 @@ let no_names dec =
   match Ast0.unwrap dec with
     Ast0.Decl(info,proto) ->
       (match Ast0.unwrap proto with
-	Ast0.FunProto(fninfo,name,lp,params,va,rp,sem) ->
+	Ast0.FunProto(fninfo,attr,name,lp,params,va,rp,sem) ->
 	  let sem =
 	    (* convert semicolon to minus, since we are dropping the whole
 	       thing *)
@@ -402,7 +402,7 @@ let no_names dec =
 	       (info,
 		Ast0.rewrap proto
 		  (Ast0.FunProto
-		     (fninfo,name,lp,
+		     (fninfo,attr,name,lp,
 		      Ast0.rewrap params
 			(let info = Ast0.get_info params in
 			let mcodekind =
@@ -449,7 +449,7 @@ let reinsert mdefs minus =
     List.map
       (function x ->
 	match Ast0.unwrap x with
-	  Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
+	  Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,_) ->
 	    (name,x)
 	| _ -> failwith "bad mdef")
       mdefs in
@@ -458,7 +458,7 @@ let reinsert mdefs minus =
       match Ast0.unwrap x with
 	Ast0.NONDECL(stmt) ->
 	  (match Ast0.unwrap stmt with
-	    Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
+	    Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,_) ->
 	      (try Ast0.rewrap x (Ast0.NONDECL(List.assoc name table))
 	      with Not_found -> x)
 	  | _ -> x)
@@ -466,7 +466,7 @@ let reinsert mdefs minus =
 	  (match Ast0.unwrap rule_elem_dots with
 	    [f] ->
 	      (match Ast0.unwrap f with
-		Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,
+		Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,attrs,lbrace,body,
 			     rbrace,_) ->
 		  (try
 		    Ast0.rewrap x
