@@ -529,10 +529,17 @@ let do_get_constants constants keywords env (neg_pos,_) =
     | Ast.MetaType(name,_,_,_) -> bind (minherited name) (k ty)
     | _ -> k ty in
 
+  let attr_arg r k a =
+    match Ast.unwrap a with
+      Ast.MetaAttr(name,_,_,_) -> bind (k a) (minherited name)
+    | Ast.MacroAttr(name) -> constants (Ast.unwrap_mcode name)
+    | Ast.MacroAttrArgs(attr,_,_,_) ->  constants (Ast.unwrap_mcode attr) in
+
   let attribute r k a =
     match Ast.unwrap a with
-      Ast.MetaAttribute(name,_,_,_) -> bind (k a) (minherited name)
-    | Ast.Attribute(attr) -> constants (Ast.unwrap_mcode attr) in
+      Ast.Attribute(attr) -> k a
+    | Ast.GccAttribute(attr_,_,_,arg,_,_) ->
+        bind (keywords (Ast.unwrap_mcode attr_)) (k a) in
 
   let declaration r k d =
     match Ast.unwrap d with
@@ -542,9 +549,9 @@ let do_get_constants constants keywords env (neg_pos,_) =
 	disj_union_all (List.map r.V.combiner_declaration decls)
     | Ast.OptDecl(decl) -> option_default
     (* need things with explicit names too *)
-    | Ast.Init(_,_,_,attr,_,_,_) | Ast.UnInit(_,_,_,attr,_) ->
+    | Ast.Init(_,_,midattr,_,endattr,_,_,_) | Ast.UnInit(_,_,midattr,_,endattr,_) ->
 	List.fold_left bind (k d)
-	  (List.map r.V.combiner_attribute attr)
+	  (List.map r.V.combiner_attribute (midattr@endattr))
     | _ -> k d in
 
   let field r k d =
@@ -646,8 +653,8 @@ let do_get_constants constants keywords env (neg_pos,_) =
     donothing donothing donothing donothing donothing donothing donothing
     ident expression string_fragment string_format donothing donothing
     fullType typeC initialiser parameter define_parameter declaration donothing
-    field ann_field donothing rule_elem statement donothing attribute donothing
-    donothing
+    field ann_field donothing rule_elem statement donothing attribute attr_arg
+    donothing donothing
 
 (* ------------------------------------------------------------------------ *)
 
@@ -694,7 +701,7 @@ let all_context =
 
   let rule_elem r k e =
     match Ast.unwrap e with
-      Ast.FunHeader(bef,_,_,_,_,_,_,_) -> bind (process_mcodekind bef) (k e)
+      Ast.FunHeader(bef,_,_,_,_,_,_,_,_) -> bind (process_mcodekind bef) (k e)
     | Ast.Decl decl ->
 	bind (process_mcodekind (annotated_decl decl)) (k e)
     | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
@@ -717,6 +724,7 @@ let all_context =
     donothing
     initialiser donothing donothing donothing donothing donothing donothing
     donothing rule_elem statement donothing donothing donothing donothing
+    donothing
 
 (* ------------------------------------------------------------------------ *)
 
