@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -10,6 +10,8 @@ type combine =
     And of combine list | Or of combine list | Not of combine
   | Elem of string | False | True
 
+val dep2c : combine -> string
+
 val get_constants :
     Ast_cocci.rule list ->
       (((Ast_cocci.meta_name list * Ast_cocci.meta_name list)
