@@ -5,8 +5,8 @@
  *)
 
 (* create an index for each constructor *)
-(* current max is 192, also unused: 8-9, 15, 42, 46, 57, 65, 85-86,
- 113-115, 140, 162 *)
+(* current max is 193, also unused: 8-9, 15, 42, 46, 57, 65, 85-86,
+ 113-115 *)
 
 (* doesn't really work - requires that identical terms with no token
 subterms (eg dots) not appear on the same line *)
@@ -114,9 +114,9 @@ let typeC t =
 let declaration d =
   match Ast0.unwrap d with
     Ast0.MetaDecl(name,_,_) -> [148]
-  | Ast0.Init(stg,ty,id,attr,eq,exp,sem) -> [54]
-  | Ast0.UnInit(stg,ty,id,attr,sem) -> [55]
-  | Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) -> [132]
+  | Ast0.Init(stg,ty,midattr,id,endattr,eq,exp,sem) -> [54]
+  | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) -> [55]
+  | Ast0.FunProto(fi,attr,name,lp1,params,va,rp1,sem) -> [132]
   | Ast0.MacroDecl(stg,name,lp,args,rp,attr,sem) -> [137]
   | Ast0.MacroDeclInit(stg,name,lp,args,rp,eq,ini,sem) -> [157]
   | Ast0.TyDecl(ty,attr,sem) -> [116]
@@ -158,7 +158,7 @@ let initialiser i =
 let parameterTypeDef p =
   match Ast0.unwrap p with
     Ast0.VoidParam(ty,attr) -> [59]
-  | Ast0.Param(ty,id,attr) -> [60]
+  | Ast0.Param(ty,midattr,id,attr) -> [60]
   | Ast0.MetaParam(name,_,_) -> [61]
   | Ast0.MetaParamList(name,_,_,_) -> [62]
   | Ast0.PComma(cm) -> [63]
@@ -168,7 +168,7 @@ let parameterTypeDef p =
 
 let statement s =
   match Ast0.unwrap s with
-    Ast0.FunDecl(bef,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,aft) -> [68]
+    Ast0.FunDecl(bef,fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,aft) -> [68]
   | Ast0.Decl(bef,decl) -> [69]
   | Ast0.Seq(lbrace,body,rbrace) -> [70]
   | Ast0.ExprStatement(exp,sem) -> [71]
@@ -232,7 +232,13 @@ let string_fragment f =
 let attribute a =
   match Ast0.unwrap a with
     Ast0.Attribute(attr) -> [39]
-  | Ast0.MetaAttribute(name,_,_) -> [40]
+  | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) -> [162]
+
+let attr_arg a =
+  match Ast0.unwrap a with
+    Ast0.MetaAttr(name,_,_) -> [40]
+  | Ast0.MacroAttr(attr) -> [140]
+  | Ast0.MacroAttrArgs(_) -> [193]
 
 let top_level t =
   match Ast0.unwrap t with
