@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -20,7 +20,7 @@ let check_parameter s =
       (try
 	let _ = int_of_string a in
 	(Some a, String.concat "$" rest)
-      with Failure "int_of_string" -> (None, s))
+      with Failure _(*"int_of_string"*) -> (None, s))
   | _ -> (None, s)
 
 let check_flags s =
