@@ -43,7 +43,7 @@ let all_removed_recursor =
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
     do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
-    do_nothing do_nothing do_nothing do_nothing do_nothing
+    do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
 
 let all_removed_decl =
   all_removed_recursor.V.combiner_declaration
@@ -79,7 +79,7 @@ let contains_modif =
        and the expression could contain a statement. *)
     let res = k re in
     match Ast.unwrap re with
-      Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp) ->
+      Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp,attrs) ->
 	bind (mcode r ((),(),bef,[])) res
     | Ast.Decl decl ->
 	bind (mcode r ((),(),annotated_decl decl,[])) res
@@ -100,13 +100,19 @@ let contains_modif =
       do_nothing do_nothing do_nothing init do_nothing
       do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing rule_elem do_nothing do_nothing do_nothing
-      do_nothing do_nothing in
+      do_nothing do_nothing do_nothing in
   recursor.V.combiner_fullType
 
+let attr_arg a =
+  match Ast.unwrap a with
+    Ast.MacroAttr(attr) -> mcode () attr
+  | Ast.MetaAttr(name,b,c,d) -> mcode () name
+  | Ast.MacroAttrArgs(attr,lp,args,rp) -> mcode () attr
+
 let attribute a =
   match Ast.unwrap a with
-    Ast.Attribute(attr) -> mcode () attr
-  | Ast.MetaAttribute(name,b,c,d) -> mcode () name
+    Ast.Attribute(attr) -> attr_arg attr
+  | Ast.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) -> attr_arg arg
 
 let decl r k e =
   let e = k e in
@@ -115,16 +121,17 @@ let decl r k e =
   | Ast.NoStorage -> {e with Ast.safe_for_multi_decls = Ast.NoStorage}
   | Ast.Unsafe ->
       match Ast.unwrap e with
-	Ast.Init(stg,ty,_,attr,_,_,sem)
-      | Ast.UnInit(stg,ty,_,attr,sem) ->
+	Ast.Init(stg,ty,midattr,_,endattr,_,_,sem)
+      | Ast.UnInit(stg,ty,midattr,_,endattr,sem) ->
 	  let stg_modif =
 	    match stg with
 	      Some stg -> mcode () stg
 	    | None -> false in
-	  let attr_modif = List.exists attribute attr in
+	  let midattr_modif = List.exists attribute midattr in
+	  let endattr_modif = List.exists attribute endattr in
 	  let ft_modif = contains_modif ty in
 	  let sem_modif = mcode () sem in
-	  if not(stg_modif || attr_modif || ft_modif || sem_modif)
+	  if not(stg_modif || midattr_modif || endattr_modif || ft_modif || sem_modif)
 	  then {e with Ast.safe_for_multi_decls = Ast.Safe}
 	  else e
       | _ -> e
@@ -179,7 +186,8 @@ let process =
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing donothing donothing donothing
       donothing donothing donothing donothing decl anndecl field annfield
-      donothing donothing donothing donothing donothing donothing donothing in
+      donothing donothing donothing donothing donothing donothing donothing
+      donothing in
   List.map fn.V.rebuilder_top_level
 
 let safe_for_multi_decls rules =
