@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -18,7 +18,7 @@ module Ast0 = Ast0_cocci
 
 let top_dots l = Ast0.wrap l
 
-let rec is_decl s =
+let is_decl s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
   | _ -> false
@@ -29,10 +29,11 @@ let isall f l = List.for_all (isonly f) l
 let rec is_toplevel s =
   match Ast0.unwrap s with
     Ast0.Decl(_,e) -> true
-  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_) -> true
+  | Ast0.FunDecl(_,_,_,_,_,_,_,_,_,_,_,_) -> true
   | Ast0.Disj(_,stmts,_,_) -> isall is_toplevel stmts
   | Ast0.ExprStatement(Some fc,_) -> false
   | Ast0.Include(_,_) -> true
+  | Ast0.MetaInclude(_,_) -> true
   | Ast0.Undef(_,_) -> true
   | Ast0.Define(_,_,_,_) -> true
   | Ast0.Pragma(_,_,_) -> true
