@@ -322,7 +322,7 @@ and typeC t =
     (function _ ->
       match Ast0.unwrap t with
 	Ast0.ConstVol(cv,ty) ->
-	  mcode U.const_vol cv; print_string " "; typeC ty
+	  List.iter (function s -> mcode U.const_vol s) cv; print_string " "; typeC ty
       |	Ast0.BaseType(ty,strings) ->
 	  List.iter (function s -> mcode print_string s; print_string " ")
 	    strings
@@ -380,7 +380,7 @@ and typeC t =
 (* Even if the Cocci program specifies a list of declarations, they are
    split out into multiple declarations of a single variable each. *)
 
-and print_named_type ty id =
+and print_named_type ty midattr id =
   match Ast0.unwrap ty with
     Ast0.Array(ty,lb,size,rb) ->
       let rec loop ty k =
@@ -392,11 +392,12 @@ and print_named_type ty id =
 		mcode print_string lb;
 		print_option expression size;
 		mcode print_string rb)
-	| _ -> typeC ty; ident id; k () in
+        | _ -> typeC ty; print_attribute_list midattr ~befspace:false ~aftspace:true; ident id; k () in
       loop ty (function _ -> ())
   | Ast0.ParenType(lp,ty,rp) ->
       print_parentype (lp,ty,rp) (function _ -> ident id)
-  | _ -> typeC ty; ident id
+  | _ -> typeC ty; print_attribute_list midattr ~befspace:false ~aftspace:true; ident id
+
 
 and declaration d =
   print_context d
@@ -404,19 +405,20 @@ and declaration d =
       match Ast0.unwrap d with
 	Ast0.MetaDecl(name,_,_) ->
 	  mcode print_meta name
-      |	Ast0.Init(stg,ty,id,attr,eq,ini,sem) ->
+      |	Ast0.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
 	  print_option (mcode U.storage) stg;
-	  print_named_type ty id;
-          print_attribute_list attr;
+	  print_named_type ty midattr id;
+          print_attribute_list endattr;
 	  print_string " ";
 	  mcode print_string eq; print_string " "; initialiser ini;
 	  mcode print_string sem
-      | Ast0.UnInit(stg,ty,id,attr,sem) ->
-	  print_option (mcode U.storage) stg; print_named_type ty id;
-          print_attribute_list attr;
+      | Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
+	  print_option (mcode U.storage) stg; print_named_type ty midattr id;
+          print_attribute_list endattr;
 	  mcode print_string sem
-      | Ast0.FunProto(fninfo,name,lp1,params,va,rp1,sem) ->
+      | Ast0.FunProto(fninfo,attr,name,lp1,params,va,rp1,sem) ->
 	  List.iter print_fninfo fninfo;
+          print_attribute_list attr ~befspace:false ~aftspace:true;
 	  ident name; mcode print_string_box lp1;
 	  parameter_list params; varargs va;
 	  close_box(); mcode print_string rp1;
@@ -467,7 +469,7 @@ and field d =
 	  begin
 	    match id with
 	      None -> typeC ty
-	    | Some id -> print_named_type ty id
+            | Some id -> print_named_type ty [] id
 	  end;
 	  let bitfield (c, e) =
 	    mcode print_string c;
@@ -553,11 +555,12 @@ and parameterTypeDef p =
         Ast0.VoidParam(ty,attr) ->
           typeC ty;
           print_attribute_list attr;
-      | Ast0.Param(ty,Some id,attr) ->
-          print_named_type ty id;
+      | Ast0.Param(ty,midattr,Some id,attr) ->
+          print_named_type ty midattr id;
           print_attribute_list attr;
-      | Ast0.Param(ty,None,attr) ->
+      | Ast0.Param(ty,midattr,None,attr) ->
           typeC ty;
+          assert (midattr = []);
           print_attribute_list attr;
       | Ast0.MetaParam(name,_,_) -> mcode print_meta name
       | Ast0.MetaParamList(name,_,_,_) -> mcode print_meta name
@@ -579,12 +582,12 @@ and statement arity s =
   print_context s
     (function _ ->
       match Ast0.unwrap s with
-	Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,lbrace,body,rbrace,_) ->
+	Ast0.FunDecl(_,fninfo,name,lp,params,va,rp,attrs,lbrace,body,rbrace,_) ->
 	  print_string arity;
 	  List.iter print_fninfo fninfo;
 	  ident name; mcode print_string_box lp;
 	  parameter_list params; varargs va; close_box(); mcode print_string rp;
-	  print_string " ";
+	  print_attribute_list attrs; print_string " ";
 	  print_string arity; mcode print_string lbrace; start_block();
 	  dots force_newline (statement arity) body;
 	  end_block(); print_string arity; mcode print_string rbrace
@@ -761,14 +764,29 @@ and print_fninfo = function
   | Ast0.FInline(inline) -> mcode print_string inline
   | Ast0.FAttr(attr) -> print_attribute attr
 
-and print_attribute_list attrs =
-  if not (attrs = []) then print_string " ";
-  print_between (fun _ -> print_string " ") print_attribute attrs
+and print_attribute_list ?(befspace=true) ?(aftspace=false) attrs =
+  if befspace && not (attrs = []) then print_string " ";
+  print_between (fun _ -> print_string " ") print_attribute attrs;
+  if aftspace && not (attrs = []) then print_string " "
 
 and print_attribute a =
   match Ast0.unwrap a with
-    Ast0.Attribute(attr) -> mcode print_string attr
-  | Ast0.MetaAttribute(name,_,_) -> mcode print_meta name
+    Ast0.Attribute(attr) -> print_attr_arg attr
+  | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+      mcode print_string attr_; print_string " ";
+      mcode print_string_box lp1; mcode print_string_box lp2;
+      print_attr_arg arg; close_box();
+      mcode print_string_box rp1; close_box();
+      mcode print_string_box rp2;
+
+and print_attr_arg a =
+  match Ast0.unwrap a with
+    Ast0.MacroAttr(arg) -> mcode print_string arg
+  | Ast0.MetaAttr(name,_,_) -> mcode print_meta name
+  | Ast0.MacroAttrArgs(attr,lp,args,rp) ->
+      mcode print_string attr; mcode print_string_box lp;
+      let _ = dots (function _ -> ()) expression args in
+      close_box(); mcode print_string rp
 
 and whencode notfn alwaysfn = function
     Ast0.WhenNot (_,_,a) ->
@@ -875,6 +893,7 @@ let rec unparse_anything x =
   | Ast0.CaseLineTag(d)  -> case_line "" d
   | Ast0.StringFragmentTag(d)  -> string_fragment d
   | Ast0.AttributeTag(d) -> print_attribute d
+  | Ast0.AttrArgTag(d)   -> print_attr_arg d
   | Ast0.TopTag(d)       -> top_level d
   | Ast0.IsoWhenTag(x)   -> U.print_when_modif x
   | Ast0.IsoWhenTTag(e)  -> expression e
