@@ -24,6 +24,7 @@ type 'a combiner =
      combiner_statement : Ast_cocci.statement -> 'a;
      combiner_case_line : Ast_cocci.case_line -> 'a;
      combiner_attribute : Ast_cocci.attr -> 'a;
+     combiner_attr_arg : Ast_cocci.attr_arg -> 'a;
      combiner_top_level : Ast_cocci.top_level -> 'a;
      combiner_anything : Ast_cocci.anything  -> 'a;
      combiner_expression_dots : Ast_cocci.expression Ast_cocci.dots -> 'a;
@@ -79,6 +80,7 @@ val combiner :
       ((Ast_cocci.statement,'a) ccode) ->
       ((Ast_cocci.case_line,'a) ccode) ->
       ((Ast_cocci.attr,'a) ccode) ->
+      ((Ast_cocci.attr_arg,'a) ccode) ->
       ((Ast_cocci.top_level,'a) ccode) ->
       ((Ast_cocci.anything,'a) ccode) ->
       'a combiner
@@ -104,6 +106,7 @@ type rebuilder =
       rebuilder_statement : Ast_cocci.statement inout;
       rebuilder_case_line : Ast_cocci.case_line inout;
       rebuilder_attribute : Ast_cocci.attr inout;
+      rebuilder_attr_arg : Ast_cocci.attr_arg inout;
       rebuilder_rule_elem : Ast_cocci.rule_elem inout;
       rebuilder_top_level : Ast_cocci.top_level inout;
       rebuilder_expression_dots : Ast_cocci.expression Ast_cocci.dots inout;
@@ -162,6 +165,7 @@ val rebuilder :
     (Ast_cocci.statement rcode) ->
     (Ast_cocci.case_line rcode) ->
     (Ast_cocci.attr rcode) ->
+    (Ast_cocci.attr_arg rcode) ->
     (Ast_cocci.top_level rcode) ->
     (Ast_cocci.anything rcode) ->
     rebuilder
