@@ -25,7 +25,7 @@ let visitor mode bind option_default
     dotsenumdeclfn dotscasefn dotsdefparfn
     identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn fieldfn
     enumdeclfn
-    stmtfn forinfofn casefn string_fragmentfn attributefn topfn =
+    stmtfn forinfofn casefn string_fragmentfn attributefn attr_argfn topfn =
   let multibind l =
     let rec loop = function
 	[] -> option_default
@@ -321,13 +321,13 @@ let visitor mode bind option_default
       rewrap t
 	(match Ast0.unwrap t with
 	  Ast0.ConstVol(cv,ty) ->
-	    let (cv_n,cv) = cv_mcode cv in
+	    let (cv_n,cv) = map_split cv_mcode cv in
 	    let (ty_n,ty) = typeC ty in
 	    let front =
 	      (* bind in the right order *)
 	      match Ast0.unwrap ty with
-		Ast0.Pointer(ty,star) -> bind ty_n cv_n
-	      |	_ -> bind cv_n ty_n in
+		Ast0.Pointer(ty,star) -> multibind (ty_n::cv_n)
+              | _ -> multibind (cv_n@[ty_n]) in
 	    (front, Ast0.ConstVol(cv,ty))
 	| Ast0.BaseType(ty,strings) ->
 	    let (strings_n,strings) = map_split_bind string_mcode strings in
@@ -342,12 +342,12 @@ let visitor mode bind option_default
 	    (bind ty_n star_n, Ast0.Pointer(ty,star))
         | Ast0.ParenType(lp,ty,rp) ->
 	    let (t,id) =
-              parentype_type (lp,ty,None,rp) in t
+              parentype_type (lp,ty,[],None,rp) in t
         | Ast0.FunctionType(ty,lp,params,rp) ->
 	    let (t,id) =
               functiontype_type (ty,None,lp,params,rp) in t
 	| Ast0.Array(ty,lb,size,rb) ->
-            let (t,id) = array_type (ty,None,lb,size,rb) in t
+            let ((t,mid),id) = array_type (ty,[],None,lb,size,rb) in t
 	| Ast0.Decimal(dec,lp,length,comma,precision_opt,rp) ->
 	    let (dec_n,dec) = string_mcode dec in
 	    let (lp_n,lp) = string_mcode lp in
@@ -417,24 +417,26 @@ let visitor mode bind option_default
     tyfn all_functions k t
 
   (* returns ((bind value,original value),id) since id may have been updated*)
-  and array_type (ty,(id : Ast0.ident option),lb,size,rb) =
+  and array_type (ty,midattr,(id : Ast0.ident option),lb,size,rb) =
     let (ty_n,ty) = typeC ty in
+    let (midattr_n,midattr) = map_split_bind attribute midattr in
     let (idl,idu) = (match id with
       | Some a -> let (b,c) = ident a in ([b],Some c)
       | None -> ([],None)) in
     let (lb_n,lb) = string_mcode lb in
     let (size_n,size) = get_option expression size in
     let (rb_n,rb) = string_mcode rb in
-    ((multibind ([ty_n] @ idl @ [lb_n;size_n;rb_n]),
-     Ast0.Array(ty,lb,size,rb)), idu)
+    (((multibind ([ty_n] @ [midattr_n] @ idl @ [lb_n;size_n;rb_n]),
+     Ast0.Array(ty,lb,size,rb)),midattr), idu)
 
-  and parentype_type (lp,ty,(id : Ast0.ident option),rp) =
+  and parentype_type (lp,ty,midattr,(id : Ast0.ident option),rp) =
     let function_pointer ty1 array_decs =
       match Ast0.unwrap ty1 with
         Ast0.Pointer(ty2,star) ->
           (match Ast0.unwrap ty2 with
             Ast0.FunctionType(ty3,lp3,params,rp3) ->
               let (ty_n,typ) = typeC ty3 in
+              let (midattr_n,midattr) = map_split_bind attribute midattr in
               let (lp_n,lp) = string_mcode lp in
               let (star_n,star) = string_mcode star in
               let (idl,idu) =
@@ -455,7 +457,7 @@ let visitor mode bind option_default
               let (params_n,params) = parameter_dots params in
               let (rp3_n,rp3) = string_mcode rp3 in
               let bind_val =
-                multibind ([ty_n;lp_n;star_n]
+                multibind ([ty_n;midattr_n;lp_n;star_n]
                 @ idl @ array_n @ [rp_n;lp3_n;params_n;rp3_n]) in
               let inner_type =
                 let inner_type1 =
@@ -490,23 +492,25 @@ let visitor mode bind option_default
     ((multibind ([ty_n] @ idl @ [lp_n; params_n; rp_n]),
      Ast0.FunctionType(ty,lp,params,rp)), idu)
 
-  and named_type ty id =
+  and named_type ty midattr id =
     match Ast0.unwrap ty with
       Ast0.Array(rty,lb,size,rb) ->
-	let (tyres, idn) = array_type (rty,Some id,lb,size,rb) in
+	let ((tyres, midattr_n), idn) = array_type (rty,midattr,Some id,lb,size,rb) in
         let idn = match idn with Some i -> i | None -> failwith "Impossible" in
-	(rewrap ty tyres, idn)
+	((rewrap ty tyres, midattr_n), idn)
     | Ast0.ParenType(lp,rty,rp) ->
-	let (tyres, idn) = parentype_type (lp,rty,Some id,rp) in
+	let (tyres, idn) = parentype_type (lp,rty,midattr,Some id,rp) in
 	let idn = match idn with Some i -> i | None -> failwith "Impossible" in
-	(rewrap ty tyres, idn)
+        ((rewrap ty tyres,[]), idn)
     | Ast0.FunctionType(rty,lp,params,rp) ->
+        assert (midattr = []); (* Impossible *)
 	let (tyres, idn) = functiontype_type (rty,Some id,lp,params,rp) in
 	let idn = match idn with Some i -> i | None -> failwith "Impossible" in
-	(rewrap ty tyres, idn)
+        ((rewrap ty tyres,[]), idn)
     | _ -> let (ty_n,ty) = typeC ty in
+           let (midattr_n,midattr) = map_split_bind attribute midattr in
            let (id_n,id) = ident id in
-             ((bind ty_n id_n, ty), id)
+           ((((multibind [ty_n;midattr_n;id_n]), ty), midattr), id)
 
   (* returns ((bind value,original value),id) since id may have been updated*)
   and array_type_typedef (ty,id,lb,size,rb) =
@@ -608,24 +612,25 @@ let visitor mode bind option_default
 	  Ast0.MetaDecl(name,constraints,pure) ->
 	    let (n,name) = meta_mcode name in
 	    (n,Ast0.MetaDecl(name,constraints,pure))
-	| Ast0.Init(stg,ty,id,attr,eq,ini,sem) ->
+	| Ast0.Init(stg,ty,midattr,id,endattr,eq,ini,sem) ->
 	    let (stg_n,stg) = get_option storage_mcode stg in
-	    let ((ty_id_n,ty),id) = named_type ty id in
-	    let (attr_n,attr) = map_split_bind attribute attr in
+	    let (((ty_ma_id_n,ty),midattr),id) = named_type ty midattr id in
+	    let (endattr_n,endattr) = map_split_bind attribute endattr in
 	    let (eq_n,eq) = string_mcode eq in
 	    let (ini_n,ini) = initialiser ini in
 	    let (sem_n,sem) = string_mcode sem in
-	    (multibind [stg_n;ty_id_n;attr_n;eq_n;ini_n;sem_n],
-	     Ast0.Init(stg,ty,id,attr,eq,ini,sem))
-	| Ast0.UnInit(stg,ty,id,attr,sem) ->
+	    (multibind [stg_n;ty_ma_id_n;endattr_n;eq_n;ini_n;sem_n],
+	     Ast0.Init(stg,ty,midattr,id,endattr,eq,ini,sem))
+	| Ast0.UnInit(stg,ty,midattr,id,endattr,sem) ->
 	    let (stg_n,stg) = get_option storage_mcode stg in
-	    let ((ty_id_n,ty),id) = named_type ty id in
-	    let (attr_n,attr) = map_split_bind attribute attr in
+	    let (((ty_ma_id_n,ty),midattr),id) = named_type ty midattr id in
+	    let (endattr_n,endattr) = map_split_bind attribute endattr in
 	    let (sem_n,sem) = string_mcode sem in
-	    (multibind [stg_n;ty_id_n;attr_n;sem_n],
-	     Ast0.UnInit(stg,ty,id,attr,sem))
-	| Ast0.FunProto(fi,name,lp1,params,va,rp1,sem) ->
+            (multibind [stg_n;ty_ma_id_n;endattr_n;sem_n],
+	     Ast0.UnInit(stg,ty,midattr,id,endattr,sem))
+	| Ast0.FunProto(fi,attr,name,lp1,params,va,rp1,sem) ->
 	    let (fi_n,fi) = map_split_bind fninfo fi in
+	    let (attr_n,attr) = map_split_bind attribute attr in
 	    let (name_n,name) = ident name in
 	    let (lp1_n,lp1) = string_mcode lp1 in
 	    let (params_n,params) = parameter_dots params in
@@ -637,8 +642,8 @@ let visitor mode bind option_default
 	        (multibind [comma_n; ellipsis_n],Some(comma,ellipsis)) end in
 	    let (rp1_n,rp1) = string_mcode rp1 in
 	    let (sem_n,sem) = string_mcode sem in
-	    (multibind [fi_n;name_n;lp1_n;params_n;va_n;rp1_n;sem_n],
-	     Ast0.FunProto(fi,name,lp1,params,va,rp1,sem))
+            (multibind [fi_n;attr_n;name_n;lp1_n;params_n;va_n;rp1_n;sem_n],
+	     Ast0.FunProto(fi,attr,name,lp1,params,va,rp1,sem))
 	| Ast0.MacroDecl(stg,name,lp,args,rp,attr,sem) ->
 	    let (stg_n,stg) = get_option storage_mcode stg in
 	    let (name_n,name) = ident name in
@@ -701,7 +706,7 @@ let visitor mode bind option_default
 	      match id with
 		None -> (typeC ty, None)
 	      | Some id ->
-		  let (ty, id) = named_type ty id in
+                  let ((ty,_), id) = named_type ty [] id in
 		  (ty, Some id) in
 	    let bitfield (c, e) =
 	      let (c_n, c) = string_mcode c in
@@ -834,14 +839,15 @@ let visitor mode bind option_default
 	    let (ty_n,ty) = typeC ty in
 	    let (attr_n,attr) = map_split_bind attribute attrs in
             (bind ty_n attr_n,Ast0.VoidParam(ty, attrs))
-	| Ast0.Param(ty,Some id,attrs) ->
-	    let ((ty_id_n,ty),id) = named_type ty id in
+	| Ast0.Param(ty,midattrs,Some id,attrs) ->
+            let (((ty_mid_id_n,ty),midattr),id) = named_type ty midattrs id in
 	    let (attr_n,attr) = map_split_bind attribute attrs in
-	    (bind ty_id_n attr_n, Ast0.Param(ty,Some id,attr))
-	| Ast0.Param(ty,None,attrs) ->
+	    (bind ty_mid_id_n attr_n, Ast0.Param(ty,midattr,Some id,attr))
+	| Ast0.Param(ty,midattrs,None,attrs) ->
 	    let (ty_n,ty) = typeC ty in
 	    let (attr_n,attr) = map_split_bind attribute attrs in
-	    (bind ty_n attr_n, Ast0.Param(ty,None,attr))
+            assert (midattrs = []);
+            (bind ty_n attr_n, Ast0.Param(ty,[],None,attr))
 	| Ast0.MetaParam(name,constraints,pure) ->
 	    let (n,name) = meta_mcode name in
 	    (n,Ast0.MetaParam(name,constraints,pure))
@@ -879,7 +885,7 @@ let visitor mode bind option_default
     let k s =
       rewrap s
 	(match Ast0.unwrap s with
-	  Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft) ->
+	  Ast0.FunDecl(bef,fi,name,lp,params,va,rp,attrs,lbrace,body,rbrace,aft) ->
 	    let (fi_n,fi) = map_split_bind fninfo fi in
 	    let (name_n,name) = ident name in
 	    let (lp_n,lp) = string_mcode lp in
@@ -891,12 +897,13 @@ let visitor mode bind option_default
                 let (ellipsis_n, ellipsis) = string_mcode ellipsis in
                 (multibind [comma_n;ellipsis_n],Some(comma,ellipsis)) in
 	    let (rp_n,rp) = string_mcode rp in
+	    let (attr_n,attr) = map_split_bind attribute attrs in
 	    let (lbrace_n,lbrace) = string_mcode lbrace in
 	    let (body_n,body) = statement_dots body in
 	    let (rbrace_n,rbrace) = string_mcode rbrace in
 	    (multibind
-	       [fi_n;name_n;lp_n;params_n;va_n;rp_n;lbrace_n;body_n;rbrace_n],
-	     Ast0.FunDecl(bef,fi,name,lp,params,va,rp,lbrace,body,rbrace,aft))
+               [fi_n;name_n;lp_n;params_n;va_n;rp_n;attr_n;lbrace_n;body_n;rbrace_n],
+	     Ast0.FunDecl(bef,fi,name,lp,params,va,rp,attr,lbrace,body,rbrace,aft))
 	| Ast0.Decl(bef,decl) ->
 	    let (decl_n,decl) = declaration decl in
 	    (decl_n,Ast0.Decl(bef,decl))
@@ -1153,12 +1160,35 @@ let visitor mode bind option_default
       rewrap a
         (match Ast0.unwrap a with
           Ast0.Attribute(attr) ->
-            let (attr_n,attr) = string_mcode attr in
+            let (attr_n,attr) = attr_arg attr in
             (attr_n,Ast0.Attribute(attr))
-	| Ast0.MetaAttribute(name,constraints,pure) ->
-	    let (n,name) = meta_mcode name in
-	    (n,Ast0.MetaAttribute(name,constraints,pure))) in
-    attributefn all_functions k a
+        | Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2) ->
+            let (attr_n,attr_) = string_mcode attr_ in
+            let (lp1_n,lp1) = string_mcode lp1 in
+            let (lp2_n,lp2) = string_mcode lp2 in
+            let (arg_n,arg) = attr_arg arg in
+            let (rp1_n,rp1) = string_mcode rp1 in
+            let (rp2_n,rp2) = string_mcode rp2 in
+            (multibind [attr_n;lp1_n;lp2_n;rp1_n;rp2_n],
+            Ast0.GccAttribute(attr_,lp1,lp2,arg,rp1,rp2))) in
+              attributefn all_functions k a
+
+  and attr_arg a =
+    let k a =
+      rewrap a
+        (match Ast0.unwrap a with
+          Ast0.MetaAttr(name,constraints,pure) ->
+            let (n,name) = meta_mcode name in
+            (n,Ast0.MetaAttr(name,constraints,pure))
+        | Ast0.MacroAttr (arg) ->
+            let (args_n,arg) = string_mcode arg in (args_n,Ast0.MacroAttr(arg))
+        | Ast0.MacroAttrArgs (attr,lp,args,rp) ->
+            let (attr_n,attr) = string_mcode attr in
+	    let (lp_n,lp) = string_mcode lp in
+	    let (args_n,args) = expression_dots args in
+	    let (rp_n,rp) = string_mcode rp in
+            (multibind [attr_n;lp_n;args_n;rp_n], Ast0.MacroAttrArgs (attr,lp,args,rp))) in
+              attr_argfn all_functions k a
 
   (* we only include the when string mcode w because the parameterised
      string_mcodefn function might have side-effects *)
@@ -1334,6 +1364,9 @@ let visitor mode bind option_default
       | Ast0.AttributeTag(a) ->
 	  let (a_n,a) = attribute a in
 	  (a_n,Ast0.AttributeTag(a))
+      | Ast0.AttrArgTag(a) ->
+	  let (a_n,a) = attr_arg a in
+	  (a_n,Ast0.AttrArgTag(a))
       | Ast0.TopTag(top) ->
 	  let (top_n,top) = top_level top in
 	  (top_n,Ast0.TopTag(top))
@@ -1377,6 +1410,7 @@ let visitor mode bind option_default
       VT0.define_param = define_param;
       VT0.string_fragment = string_fragment;
       VT0.attribute = attribute;
+      VT0.attr_arg = attr_arg;
       VT0.top_level = top_level;
       VT0.expression_dots = expression_dots;
       VT0.statement_dots = statement_dots;
@@ -1427,6 +1461,7 @@ let combiner_functions =
    VT0.combiner_casefn = (fun r k e -> k e);
    VT0.combiner_string_fragmentfn = (fun r k e -> k e);
    VT0.combiner_attributefn = (fun r k e -> k e);
+   VT0.combiner_attr_argfn = (fun r k e -> k e);
    VT0.combiner_topfn = (fun r k e -> k e)}
 
 let combiner_dz r =
@@ -1468,6 +1503,8 @@ let combiner_dz r =
       (function e -> let (n,_) = r.VT0.string_fragment e in n);
       VT0.combiner_rec_attribute =
       (function e -> let (n,_) = r.VT0.attribute e in n);
+      VT0.combiner_rec_attr_arg =
+      (function e -> let (n,_) = r.VT0.attr_arg e in n);
       VT0.combiner_rec_top_level =
       (function e -> let (n,_) = r.VT0.top_level e in n);
       VT0.combiner_rec_expression_dots =
@@ -1535,6 +1572,7 @@ let combiner bind option_default functions =
     (fun r k e -> (functions.VT0.combiner_casefn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_string_fragmentfn (dz r) (xk k) e,e))
     (fun r k e -> (functions.VT0.combiner_attributefn (dz r) (xk k) e, e))
+    (fun r k e -> (functions.VT0.combiner_attr_argfn (dz r) (xk k) e, e))
     (fun r k e -> (functions.VT0.combiner_topfn (dz r) (xk k) e, e)))
 
 let flat_combiner bind option_default
@@ -1545,7 +1583,7 @@ let flat_combiner bind option_default
     dotsenumdeclfn dotscasefn dotsdefparfn
     identfn exprfn assignOpfn binaryOpfn tyfn initfn paramfn declfn fieldfn
     enumdeclfn
-    stmtfn forinfofn casefn string_fragmentfn attributefn topfn =
+    stmtfn forinfofn casefn string_fragmentfn attributefn attr_argfn topfn =
   let dz = combiner_dz in
   let xk k e = let (n,_) = k e in n in
   combiner_dz (visitor COMBINER bind option_default
@@ -1587,6 +1625,7 @@ let flat_combiner bind option_default
     (fun r k e -> (casefn (dz r) (xk k) e, e))
     (fun r k e -> (string_fragmentfn (dz r) (xk k) e, e))
     (fun r k e -> (attributefn (dz r) (xk k) e, e))
+    (fun r k e -> (attr_argfn (dz r) (xk k) e, e))
     (fun r k e -> (topfn (dz r) (xk k) e, e)))
 
 let rebuilder_functions =
@@ -1628,6 +1667,7 @@ let rebuilder_functions =
    VT0.rebuilder_casefn = (fun r k e -> k e);
    VT0.rebuilder_string_fragmentfn = (fun r k e -> k e);
    VT0.rebuilder_attributefn = (fun r k e -> k e);
+   VT0.rebuilder_attr_argfn = (fun r k e -> k e);
    VT0.rebuilder_topfn = (fun r k e -> k e)}
 
 let rebuilder_dz r =
@@ -1667,6 +1707,8 @@ let rebuilder_dz r =
       (function e -> let (_,e) = r.VT0.string_fragment e in e);
       VT0.rebuilder_rec_attribute =
       (function e -> let (_,e) = r.VT0.attribute e in e);
+      VT0.rebuilder_rec_attr_arg =
+      (function e -> let (_,e) = r.VT0.attr_arg e in e);
       VT0.rebuilder_rec_top_level =
       (function e -> let (_,e) = r.VT0.top_level e in e);
       VT0.rebuilder_rec_expression_dots =
@@ -1730,6 +1772,7 @@ let rebuilder functions =
     (fun r k e ->
       ((),functions.VT0.rebuilder_string_fragmentfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_attributefn (dz r) (xk k) e))
+    (fun r k e -> ((),functions.VT0.rebuilder_attr_argfn (dz r) (xk k) e))
     (fun r k e -> ((),functions.VT0.rebuilder_topfn (dz r) (xk k) e)))
 
 let flat_rebuilder
@@ -1741,7 +1784,7 @@ let flat_rebuilder
     dotsenumdeclfn dotscasefn dotsdefparfn
     identfn exprfn assignOpfn arithOpfn tyfn initfn paramfn declfn fieldfn
     enumdeclfn
-    stmtfn forinfofn casefn string_fragmentfn attributefn topfn =
+    stmtfn forinfofn casefn string_fragmentfn attributefn attr_argfn topfn =
   let dz = rebuilder_dz in
   let xk k e = let (_,e) = k e in e in
   rebuilder_dz
@@ -1784,6 +1827,7 @@ let flat_rebuilder
     (fun r k e -> ((),casefn (dz r) (xk k) e))
     (fun r k e -> ((),string_fragmentfn (dz r) (xk k) e))
     (fun r k e -> ((),attributefn (dz r) (xk k) e))
+    (fun r k e -> ((),attr_argfn (dz r) (xk k) e))
     (fun r k e -> ((),topfn (dz r) (xk k) e)))
 
 let combiner_rebuilder_functions =
@@ -1839,6 +1883,7 @@ let combiner_rebuilder_functions =
    VT0.combiner_rebuilder_casefn = (fun r k e -> k e);
    VT0.combiner_rebuilder_string_fragmentfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_attributefn = (fun r k e -> k e);
+   VT0.combiner_rebuilder_attr_argfn = (fun r k e -> k e);
    VT0.combiner_rebuilder_topfn = (fun r k e -> k e)}
 
 let combiner_rebuilder bind option_default functions =
@@ -1881,4 +1926,5 @@ let combiner_rebuilder bind option_default functions =
     functions.VT0.combiner_rebuilder_casefn
     functions.VT0.combiner_rebuilder_string_fragmentfn
     functions.VT0.combiner_rebuilder_attributefn
+    functions.VT0.combiner_rebuilder_attr_argfn
     functions.VT0.combiner_rebuilder_topfn
