@@ -33,7 +33,7 @@ let contains_modif =
   let rule_elem r k re =
     let res = k re in
     match Ast.unwrap re with
-      Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp) ->
+      Ast.FunHeader(bef,_,fninfo,name,lp,params,va,rp,endattrs) ->
       bind (mcode r ((),(),bef,[])) res
     | Ast.Decl decl -> bind (mcode r ((),(),annotated_decl decl,[])) res
     | Ast.ForHeader(fr,lp,Ast.ForDecl(decl),e2,sem2,e3,rp) ->
@@ -48,7 +48,7 @@ let contains_modif =
       do_nothing do_nothing do_nothing do_nothing do_nothing do_nothing
       do_nothing do_nothing do_nothing
       do_nothing do_nothing rule_elem do_nothing do_nothing do_nothing
-      do_nothing do_nothing in
+      do_nothing do_nothing do_nothing in
   recursor.V.combiner_rule_elem
 
 let ctl_exists keep_wit v x =
