@@ -298,7 +298,6 @@
 #define  fastcall
 #define  asmlinkage
 
-#define  far
 #define  SK_FAR
 
 // pb
@@ -368,6 +367,9 @@
 #define __releases(x)
 #define __acquires(x)
 
+#define __must_hold(x)
+#define __printf(a,b)
+
 //now in lexer
 //#define __declspec(x)
 
@@ -423,8 +425,8 @@ static const struct machine_desc __mach_desc_##_type	\
 // include/linux/types.h
 //#define BITS_TO_LONGS(bits) \
 //	(((bits)+BITS_PER_LONG-1)/BITS_PER_LONG)
-//#define DECLARE_BITMAP(name,bits)
-//	/*unsigned*/ long name[BITS_TO_LONGS(bits)]
+#define DECLARE_BITMAP(name,bits)
+	/*unsigned*/ long name[BITS_TO_LONGS(bits)]
 
 
 // include/asm-i386/percpu.h
