@@ -404,13 +404,13 @@ let rec declaration_pos decl snp
   | Ast0.MetaDecl _
   | Ast0.AsDecl _ ->
       None
-  | Ast0.Init(st, ty, id, attr, eq, ini, sem) ->
+  | Ast0.Init(st, ty, midattr, id, endattr, eq, ini, sem) ->
       let _ = type_pos ty snp in (* sanity check *)
-      let constructor ~id = Ast0.Init(st, ty, id, attr, eq, ini, sem) in
+      let constructor ~id = Ast0.Init(st, ty, midattr, id, endattr, eq, ini, sem) in
       id_wrap ~id ~constructor snp
-  | Ast0.UnInit(st, ty, id, attr, sem) ->
+  | Ast0.UnInit(st, ty, midattr, id, endattr, sem) ->
       let _ = type_pos ty snp in (* sanity check *)
-      let constructor ~id = Ast0.UnInit(st, ty, id, attr, sem) in
+      let constructor ~id = Ast0.UnInit(st, ty, midattr, id, endattr, sem) in
       id_wrap ~id ~constructor snp
   | Ast0.TyDecl (t, attr, sem) ->
       let c ~item ~mc = Ast0.TyDecl(item, attr, mc) in
@@ -428,8 +428,8 @@ let rec declaration_pos decl snp
   | Ast0.OptDecl(dec) ->
       let constructor ~item = Ast0.OptDecl item in
       item_wrap ~item:dec ~item_posfn:declaration_pos ~constructor snp
-  | Ast0.FunProto(fninfo,id,lp1,params,va,rp1,sem) ->
-      let constructor ~id = Ast0.FunProto(fninfo,id,lp1,params,va,rp1,sem) in
+  | Ast0.FunProto(fninfo,attr,id,lp1,params,va,rp1,sem) ->
+      let constructor ~id = Ast0.FunProto(fninfo,attr,id,lp1,params,va,rp1,sem) in
       id_wrap ~id ~constructor snp
 
 let rec field_pos decl snp
@@ -575,6 +575,6 @@ let rec statement_pos s snp
   | Ast0.Return (retmc,sem) ->
       let constructor ~mc = Ast0.Return(mc, sem) in
       mcode_wrap ~mc:retmc ~constructor snp
-  | Ast0.FunDecl (b, f, id, lp, ps, op, rp, lb, sd, rb, a) ->
-      let constructor ~id = Ast0.FunDecl(b,f,id,lp,ps,op,rp,lb,sd,rb,a) in
+  | Ast0.FunDecl (b, f, id, lp, ps, op, rp, ea, lb, sd, rb, a) ->
+      let constructor ~id = Ast0.FunDecl(b,f,id,lp,ps,op,rp,ea,lb,sd,rb,a) in
       id_wrap ~id ~constructor snp
