@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -37,6 +37,11 @@ val declaration_pos :
   Snapshot.t ->
   (Ast0_cocci.declaration * Snapshot.t) option
 
+val field_pos :
+  Ast0_cocci.field ->
+  Snapshot.t ->
+  (Ast0_cocci.field * Snapshot.t) option
+
 val statement_pos :
   Ast0_cocci.statement ->
   Snapshot.t ->
